# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Snmp_server parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def communities_tmplt(config_data):
    name = config_data.get("name", "")
    command = "snmp-server community {name}".format(name=name)
    if config_data.get("rw"):
        command += " RW"
    elif config_data.get("ro"):
        command += " RO"
    if config_data.get("sdrowner"):
        command += " SDROwner"
    elif config_data.get("systemowner"):
        command += " SystemOwner"
    if config_data.get("acl_v4"):
        command += " IPv4 {IPv4}".format(IPv4=config_data["acl_v4"])
    if config_data.get("acl_v6"):
        command += " IPv6 {IPv6}".format(IPv6=config_data["acl_v6"])
    if config_data.get("v4_acl"):
        command += " {v4_acl}".format(v4_acl=config_data["v4_acl"])
    return command


def community_maps_tmplt(config_data):
    name = config_data.get("name", "")
    command = "snmp-server community-map {name}".format(name=name)
    if config_data.get("context"):
        command += " context {context}".format(context=config_data["context"])
    if config_data.get("security_name"):
        command += " security-name {security_name}".format(
            security_name=config_data["security_name"],
        )
    if config_data.get("target_list"):
        command += " target-list {target_list}".format(
            target_list=config_data["target_list"],
        )
    return command


def tmplt_correlator_rule(config_data):
    rule_name = config_data.get("rule_name")
    command = "snmp-server correlator rule {rule_name}".format(
        rule_name=rule_name,
    )
    if config_data.get("timeout"):
        command += " timeout {timeout}".format(timeout=config_data["timeout"])
    return command


def group_tmplt(config_data):
    group = config_data.get("group", "")
    command = "snmp-server group {group}".format(group=group)
    if config_data.get("version"):
        command += " {version}".format(version=config_data["version"])
    if config_data.get("notify"):
        command += " notify {notify}".format(notify=config_data["notify"])
    if config_data.get("read"):
        command += " read {read}".format(read=config_data["read"])
    if config_data.get("write"):
        command += " write {write}".format(write=config_data["write"])
    if config_data.get("context"):
        command += " context {context}".format(context=config_data["context"])
    if config_data.get("acl_v4"):
        command += " IPv4 {acl_v4}".format(acl_v4=config_data["acl_v4"])
    if config_data.get("acl_v6"):
        command += " IPv6 {acl_v6}".format(acl_v6=config_data["acl_v6"])
    if config_data.get("v4_acl"):
        command += " {v4_acl}".format(v4_acl=config_data["v4_acl"])
    return command


def host_tmplt(config_data):
    host = config_data.get("host", "")
    command = "snmp-server host {host}".format(host=host)
    if config_data.get("informs"):
        command += " informs"
    if config_data.get("traps"):
        command += " traps"
    if config_data.get("version"):
        command += " version {version}".format(version=config_data["version"])
    if config_data.get("community"):
        command += " {community}".format(community=config_data["community"])
    if config_data.get("udp_port"):
        command += " udp-port {udp_port}".format(
            udp_port=config_data["udp_port"],
        )
    if config_data.get("write"):
        command += " write {write}".format(write=config_data["write"])
    return command


def interfaces_tmplt(config_data):
    interface = config_data.get("name", "")
    notification_linkupdown_disable = config_data.get(
        "notification_linkupdown_disable",
        "",
    )
    index_persistent = config_data.get("index_persistent", "")

    cmds = []
    if notification_linkupdown_disable:
        command = "snmp-server interface {interface} notification linkupdown disable".format(
            interface=interface,
        )
        cmds.append(command)
    if config_data.get("index_persistent"):
        command = "snmp-server snmp-server interface {interface} index persistence".format(
            interface=interface,
        )
        cmds.append(command)
    if not notification_linkupdown_disable and not index_persistent and interface:
        command = "snmp-server interface {interface}".format(
            interface=interface,
        )
        cmds.append(command)
    return cmds


def mib_schema_tmplt(config_data):
    name = config_data.get("name", "")
    object_list = config_data.get("object_list", "")
    poll_interval = config_data.get("poll_interval", "")

    cmds = []
    if object_list:
        command = "snmp-server mib bulkstat schema {name} object-list {object_list}".format(
            name=name,
            object_list=object_list,
        )
        cmds.append(command)
    if poll_interval:
        command = "snmp-server mib bulkstat schema {name} poll-interval {poll_interval}".format(
            name=name,
            poll_interval=poll_interval,
        )
        cmds.append(command)
    if not object_list and not poll_interval and name:
        command = "snmp-server mib bulkstat schema {name}".format(name=name)
        cmds.append(command)
    return cmds


def mib_bulkstat_transfer_ids_tmplt(config_data):
    name = config_data.get("name", "")
    buffer_size = config_data.get("buffer_size", "")
    enable = config_data.get("enable", "")
    format_schemaASCI = config_data.get("format_schemaASCI", "")
    retain = config_data.get("retain", "")
    retry = config_data.get("retry", "")
    schema = config_data.get("schema", "")
    transfer_interval = config_data.get("transfer_interval", "")

    cmds = []
    if buffer_size:
        command = "snmp-server mib bulkstat transfer-id {name} buffer-size {buffer_size}".format(
            name=name,
            buffer_size=buffer_size,
        )
        cmds.append(command)
    if enable:
        command = "snmp-server mib bulkstat transfer-id {name} enable".format(
            name=name,
        )
        cmds.append(command)
    if format_schemaASCI:
        command = "snmp-server mib bulkstat transfer-id {name} format schemaASCII".format(
            name=name,
        )
        cmds.append(command)
    if retain:
        command = "snmp-server mib bulkstat transfer-id {name} retain {retain}".format(
            name=name,
            retain=retain,
        )
        cmds.append(command)
    if retry:
        command = "snmp-server mib bulkstat transfer-id {name} retry {retry}".format(
            name=name,
            retry=retry,
        )
        cmds.append(command)
    if schema:
        command = "snmp-server mib bulkstat transfer-id {name} schema {schema}".format(
            name=name,
            schema=schema,
        )
        cmds.append(command)
    if transfer_interval:
        command = "snmp-server mib bulkstat transfer-id {name} transfer_interval {transfer_interval}".format(
            name=name,
            transfer_interval=transfer_interval,
        )
        cmds.append(command)
    if (
        not any(
            [
                buffer_size,
                enable,
                format_schemaASCI,
                retry,
                retain,
                schema,
                transfer_interval,
            ],
        )
        and name
    ):
        command = "snmp-server mib bulkstat transfer-id {name}".format(
            name=name,
        )
        cmds.append(command)
    return cmds


def overload_control_tmplt(config_data):
    config_data = config_data.get("overload_control", {})
    command = "snmp-server overload-control"
    if config_data.get("overload_drop_time"):
        command += " {overload_drop_time}".format(
            overload_drop_time=config_data["overload_drop_time"],
        )
    if config_data.get("overload_throttle_rate"):
        command += " {overload_throttle_rate}".format(
            overload_throttle_rate=config_data["overload_throttle_rate"],
        )
    return command


def targets_tmplt(config_data):
    name = config_data.get("name", "")
    command = ""
    if name:
        command = "snmp-server target list {name}".format(name=name)
    if config_data.get("host"):
        command += " host {host}".format(host=config_data["host"])
    if config_data.get("vrf"):
        command += " vrf {vrf}".format(vrf=config_data["vrf"])
    return command


def tmplt_traps_isis(config_data):
    isis = config_data.get("traps", {}).get("isis", {})
    command = "snmp-server traps isis"
    if isis.get("all"):
        command += " all"
    else:
        if isis.get("adjacency_change"):
            command += " adjacency-change"
        if isis.get("area_mismatch"):
            command += " area-mismatch"
        if isis.get("attempt_to_exceed_max_sequence"):
            command += " attempt-to-exceed-max-sequence"
        if isis.get("authentication_failure"):
            command += " authentication-failure"
        if isis.get("authentication_type_failure"):
            command += " authentication-type-failure"
        if isis.get("corrupted_lsp_detected"):
            command += " corrupted-lsp-detected"
        if isis.get("database_overload"):
            command += " database-overload"
        if isis.get("id_len_mismatch"):
            command += " id-len-mismatch"
        if isis.get("lsp_error_detected"):
            command += " lsp-error-detected"
        if isis.get("lsp_too_large_to_propagate"):
            command += " lsp-too-large-to-propagate"
        if isis.get("manual_address_drops"):
            command += " manual-address-drops"
        if isis.get("max_area_addresses_mismatch"):
            command += " max-area-addresses-mismatch"
        if isis.get("orig_lsp_buff_size_mismatch"):
            command += " orig-lsp-buff-size-mismatch"
        if isis.get("version_skew"):
            command += " version-skew"
        if isis.get("own_lsp_purge"):
            command += " own-lsp-purge"
        if isis.get("rejected_adjacency"):
            command += " rejected-adjacency"
        if isis.get("protocols_supported_mismatch"):
            command += " protocols-supported-mismatch"
        if isis.get("sequence_number_skip"):
            command += " sequence-number-skip"
    return command


def user_tmplt(config_data):
    user = config_data.get("user", "")
    group = config_data.get("group", "")
    version = config_data.get("version", "")
    command = "snmp-server user {user} {group} {version}".format(
        user=user,
        group=group,
        version=version,
    )
    if config_data.get("acl_v4"):
        command += " IPv4 {acl_v4}".format(acl_v4=config_data["acl_v4"])
    if config_data.get("acl_v6"):
        command += " IPv6 {acl_v6}".format(acl_v6=config_data["acl_v6"])
    if config_data.get("v4_acl"):
        command += " {v4_acl}".format(v4_acl=config_data["v4_acl"])
    if config_data.get("SDROwner"):
        command += " SDROwner"
    elif config_data.get("SystemOwner"):
        command += " SystemOwner"
    return command


class Snmp_serverTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Snmp_serverTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "chassis_id",
            "getval": re.compile(
                r"""
                ^snmp-server
                (\s+chassis-id\s(?P<chassis_id>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server chassis-id {{chassis_id}}",
            "result": {
                "chassis_id": "{{chassis_id}}",
            },
        },
        {
            "name": "communities",
            "getval": re.compile(
                r"""
                ^snmp-server\scommunity
                (\s(?P<name>\S+))?
                (\sRW(?P<rw>))?
                (\sRO(?P<ro>))?
                (\sSDROwner(?P<sdrowner>))?
                (\sSystemOwner(?P<systemowner>))?
                (\sIPv4\s(?P<ipv4>\S+))?
                (\sIPv6\s(?P<ipv6>\S+))?
                (\s(?P<v4acl>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": communities_tmplt,
            "result": {
                "communities": [
                    {
                        "name": "{{ name }}",
                        "rw": "{{ True if rw is defined }}",
                        "ro": "{{ True if ro is defined }}",
                        "acl_v4": "{{ipv4}}",
                        "acl_v6": "{{ipv6}}",
                        "sdrowner": "{{True if sdrowner is defined}}",
                        "systemowner": "{{True if systemowner is defined }}",
                        "v4_acl": "{{v4acl}}",
                    },
                ],
            },
        },
        {
            "name": "community_maps",
            "getval": re.compile(
                r"""
                ^snmp-server\scommunity-map
                (\s(?P<name>\S+))?
                (\scontext\s(?P<context>\S+))?
                (\ssecurity-name\s(?P<security_name>\S+))?
                (\starget-list\s(?P<target_list>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": community_maps_tmplt,
            "result": {
                "community_maps": [
                    {
                        "name": "{{ name }}",
                        "context": "{{context}}",
                        "security_name": "{{security_name}}",
                        "target_list": "{{target_list}}",
                    },
                ],
            },
        },
        {
            "name": "correlator.buffer_size",
            "getval": re.compile(
                r"""
                ^snmp-server\scorrelator
                (\sbuffer-size\s(?P<buffer_size>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server correlator buffer-size {{correlator.buffer_size }}",
            "result": {
                "correlator": {
                    "buffer_size": "{{ buffer_size }}",
                },
            },
        },
        {
            "name": "correlator.rules",
            "getval": re.compile(
                r"""
                ^snmp-server\scorrelator
                (\srule\s(?P<name>\S+))?
                (\s+timeout\s(?P<timeout>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_correlator_rule,
            "result": {
                "correlator": {
                    "rules": [
                        {
                            "rule_name": "{{ name }}",
                            "timeout": "{{ timeout }}",

                        },
                    ],
                },
            },
        },
        {
            "name": "correlator.rule_sets",
            "getval": re.compile(
                r"""
                ^snmp-server\scorrelator\sruleset\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server correlator ruleset {{name}}",
            "result": {
                "correlator": {
                    "rule_sets":
                        [
                            {"name": "{{ name }}"},
                        ],
                },
            },
        },
        {
            "name": "contact",
            "getval": re.compile(
                r"""
                ^snmp-server\scontact\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server contact {{contact}}",
            "result": {
                "contact": "{{name}}",
            },
        },
        {
            "name": "context",
            "getval": re.compile(
                r"""
                ^snmp-server\scontext\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server context {{name}}",
            "result": {
                "context": {
                    "{{name}}": "{{name}}",
                },
            },
        },
        {
            "name": "drop.report_IPv4",
            "getval": re.compile(
                r"""
                ^snmp-server\sdrop
                (\sreport\sacl\sIPv4\s(?P<report_IPv4>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server drop report acl IPv4 {{drop.report_IPv4}}",
            "result": {
                "drop": {
                    "report_IPv4": "{{report_IPv4}}",
                },
            },
        },
        {
            "name": "drop.report_IPv6",
            "getval": re.compile(
                r"""
                ^snmp-server\sdrop
                (\sreport\sacl\sIPv6\s(?P<report_IPv6>\S+))

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server drop report acl IPv6 {{drop.report_IPv6}}",
            "result": {
                "drop": {
                    "report_IPv6": "{{report_IPv6}}",
                },
            },
        },
        {
            "name": "drop.unknown_user",
            "getval": re.compile(
                r"""
                ^snmp-server\sdrop
                (\sunknown-user(?P<unknown_user>))

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server drop unknown-user",
            "result": {
                "drop": {
                    "unknown_user": "{{True if unknown_user is defined}}",
                },
            },
        },
        {
            "name": "groups",
            "getval": re.compile(
                r"""
                ^snmp-server
                (\sgroup\s(?P<group>\S+))
                (\s(?P<version>v1|v2c|v3))
                (\snotify\s(?P<notify>\S+))?
                (\sread\s(?P<read>\S+))?
                (\swrite\s(?P<write>\S+))?
                (\scontext\s(?P<context>\S+))?
                (\sIPv4\s(?P<IPv4>\S+))?
                (\sIPv6\s(?P<IPv6>\S+))?
                (\s(?P<v4acl>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": group_tmplt,
            "result": {
                "groups": [
                    {
                        "group": "{{ group }}",
                        "acl_v4": "{{IPv4}}",
                        "acl_v6": "{{IPv6}}",
                        "context": "{{context}}",
                        "notify": "{{notify}}",
                        "read": "{{read}}",
                        "write": "{{write}}",
                        "v4_acl": "{{v4acl}}",
                        "version": "{{version}}",
                    },
                ],
            },
        },
        {
            "name": "hosts",
            "getval": re.compile(
                r"""
                ^snmp-server(\shost\s(?P<host>\S+))
                (\s(?P<traps>traps))?
                (\s(?P<informs>informs))?
                (\sversion\s(?P<version>1|2c|3))?
                (\s(?P<community>\S+))?
                (\sudp-port\s(?P<port>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": host_tmplt,
            "result": {
                "hosts": [
                    {
                        "host": "{{ host }}",
                        "traps": "{{True if traps is defined}}",
                        "informs": "{{True if informs is defined}}",
                        "community": "{{community}}",
                        "udp_port": "{{port}}",
                        "version": "{{version}}",
                    },
                ],
            },
        },
        {
            "name": "ifindex",
            "getval": re.compile(
                r"""
                ^snmp-server(\sifindex\spersist(?P<ifindex>))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ifindex persist",
            "result": {
                "ifindex": "{{True if ifindex is defined}}",
            },
        },
        {
            "name": "ifmib.internal_cache_max_duration",
            "getval": re.compile(
                r"""
                ^snmp-server\sifmib
                (\sinternal\scache\smax-duration\s(?P<cache>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ifmib internal cache max-duration {{ifmib.internal_cache_max_duration}}",
            "result": {
                "ifmib": {
                    "internal_cache_max_duration": "{{cache}}",
                },
            },
        },
        {
            "name": "ifmib.ipsubscriber",
            "getval": re.compile(
                r"""
                ^snmp-server\sifmib
                (\sipsubscriber(?P<ipsub>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ifmib ipsubscriber",
            "result": {
                "ifmib": {
                    "internal_cache_max_duration": "{{cache}}",
                    "ipsubscriber": "{{True if ipsub is defined}}",
                    "stats": "{{True if s_cache is defined}}",
                    "ifalias_long": "{{True if long is defined}}",
                },
            },
        },
        {
            "name": "ifmib.stats",
            "getval": re.compile(
                r"""
                ^snmp-server\sifmib
                (\sstats\scache(?P<s_cache>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ifmib stats cache",
            "result": {
                "ifmib": {
                    "stats": "{{True if s_cache is defined}}",
                },
            },
        },
        {
            "name": "ifmib.ifalias_long",
            "getval": re.compile(
                r"""
                ^snmp-server\sifmib
                (\sifalias\slong(?P<long>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ifmib ifalias long",
            "result": {
                "ifmib": {
                    "ifalias_long": "{{True if long is defined}}",
                },
            },
        },
        {
            "name": "inform.pending",
            "getval": re.compile(
                r"""
                ^snmp-server\sinform
                (\spending\s(?P<pending>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server inform pending {{inform.pending}}",
            "result": {
                "inform": {
                    "pending": "{{pending}}",

                },
            },
        },
        {
            "name": "inform.retries",
            "getval": re.compile(
                r"""
                ^snmp-server\sinform
                (\sretries\s(?P<retries>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server inform retries {{inform.retries}}",
            "result": {
                "inform": {
                    "retries": "{{retries}}",
                },
            },
        },
        {
            "name": "inform.timeout",
            "getval": re.compile(
                r"""
                ^snmp-server\sinform
                (\stimeout\s(?P<timeout>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server inform pending {{inform.timeout}}",
            "result": {
                "inform": {
                    "timeout": "{{timeout}}",

                },
            },
        },
        {
            "name": "interfaces",
            "getval": re.compile(
                r"""
                ^snmp-server(\sinterface\s(?P<interface>\S+))
                (\snotification\slinkupdown\sdisable(?P<notification_linkupdown_disable>))?
                (\sndex\spersistence(?P<index_persistent>))?
                $""", re.VERBOSE,
            ),
            "setval": interfaces_tmplt,
            "result": {
                "interfaces": {
                    "{{interface}}": {
                        "name": "{{ interface }}",
                        "notification_linkupdown_disable": "{{True if notification_linkupdown_disable is defined}}",
                        "index_persistent": "{{True if index_persistent is defined}}",
                    },
                },
            },
        },
        {
            "name": "ipv4.dscp",
            "getval": re.compile(
                r"""
                ^snmp-server
                \sipv4\sdscp\s(?P<dscp>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ipv4 dscp {{ipv4.dscp}}",
            "result": {
                "ipv4": {
                    "dscp": "{{dscp}}",
                },
            },
        },
        {
            "name": "ipv6.dscp",
            "getval": re.compile(
                r"""
                ^snmp-server
                (\sipv6\sdscp\s(?P<dscp>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ipv6 dscp {{ipv6.dscp}}",
            "result": {
                "ipv6": {
                    "dscp": "{{dscp}}",
                },
            },
        },
        {
            "name": "ipv4.precedence",
            "getval": re.compile(
                r"""
                ^snmp-server
                (\sipv4\sprecedence\s(?P<precedence>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ipv4 precedence {{ipv4.precedence}}",
            "result": {
                "ipv4": {"precedence": "{{precedence}}"},
            },
        },
        {
            "name": "ipv6.precedence",
            "getval": re.compile(
                r"""
                ^snmp-server
                (\sipv6\sprecedence\s(?P<precedence>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server ipv6 precedence {{ipv6.precedence}}",
            "result": {
                "ipv6": {"precedence": "{{precedence}}"},
            },
        },
        {
            "name": "location",
            "getval": re.compile(
                r"""
                ^snmp-server(\slocation\s(?P<loc>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server location {{location}}",
            "result": {
                "location": "{{ loc }}",
            },
        },
        {
            "name": "logging_threshold_oid_processing",
            "getval": re.compile(
                r"""
                ^snmp-server(\slogging\sthreshold\soid-processing\s(?P<loc>\d+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server logging threshold oid-processing {{logging_threshold_oid_processing}}",
            "result": {
                "logging_threshold_oid_processing": "{{ loc }}",
            },
        },
        {
            "name": "logging_threshold_pdu_processing",
            "getval": re.compile(
                r"""
                ^snmp-server(\slogging\sthreshold\spdu-processing\s(?P<loc>\d+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server logging threshold pdu-processing {{logging_threshold_pdu_processing}}",
            "result": {
                "logging_threshold_pdu_processing": "{{ loc }}",
            },
        },
        {
            "name": "mib_bulkstat_max_procmem_size",
            "getval": re.compile(
                r"""
                ^snmp-server(\smib\sbulkstat\smax-procmem-size\s(?P<loc>\d+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server mib bulkstat max-procmem-size {{mib_bulkstat_max_procmem_size}}",
            "result": {
                "mib_bulkstat_max_procmem_size": "{{ loc }}",
            },
        },
        {
            "name": "mib_object_lists",
            "getval": re.compile(
                r"""
                ^snmp-server(\smib\sbulkstat\sobject-list\s(?P<o_list>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server mib bulkstat object-list {{mib_object}}",
            "result": {
                "mib_object_lists": {
                    "{{o_list}}": "{{o_list}}",
                },
            },
        },
        {
            "name": "mib_schema",
            "getval": re.compile(
                r"""
                ^snmp-server(\smib\sbulkstat\sschema\s(?P<mib>\S+))
                (\sobject-list\s(?P<o_list>\S+))?
                (\spoll-interval\s(?P<p_interval>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": mib_schema_tmplt,
            "result": {
                "mib_schema": {
                    "{{mib}}": {
                        "name": "{{ mib }}",
                        "poll_interval": "{{p_interval}}",
                        "object_list": "{{o_list}}",
                    },
                },
            },
        },
        {
            "name": "mib_bulkstat_transfer_ids",
            "getval": re.compile(
                r"""
                ^snmp-server(\smib\sbulkstat\stransfer-id\s(?P<mib1>\S+))
                (\sretry\s(?P<retry>\S+))?
                (\sbuffer-size\s(?P<buffer_size>\S+))?
                (\senable(?P<enable>))?
                (\sformat\sschemaASCII(?P<format>))?
                (\sretain\s(?P<retain>\S+))?
                (\sschema\s(?P<schema>\S+))?
                (\stransfer-interval\s(?P<ti>\S+))?

                $""", re.VERBOSE,
            ),
            "setval": mib_bulkstat_transfer_ids_tmplt,
            "result": {
                "mib_bulkstat_transfer_ids": {
                    "{{mib1}}": {
                        "name": "{{ mib1 }}",
                        "buffer_size": "{{buffer_size}}",
                        "enable": "{{True if enable is defined}}",
                        "format_schemaASCI": "{{True if format is defined}}",
                        "retain": "{{retain}}",
                        "schema": "{{schema}}",
                        "retry": "{{retry}}",
                        "transfer_interval": "{{ti}}",
                    },
                },
            },
        },
        {
            "name": "mroutemib_send_all_vrf",
            "getval": re.compile(
                r"""
                ^snmp-server(\smroutemib\ssend-all-vrf(?P<send>))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server mroutemib send-all-vrf",
            "result": {
                "mroutemib_send_all_vrf": "{{ True if send is defined }}",
            },
        },
        {
            "name": "notification_log_mib.size",
            "getval": re.compile(
                r"""
                ^snmp-server\snotification-log-mib
                (\ssize\s(?P<size>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server notification-log-mib size {{notification_log_mib.size}}",
            "result": {
                "notification_log_mib": {
                    "size": "{{size}}",
                },
            },
        },
        {
            "name": "notification_log_mib.default",
            "getval": re.compile(
                r"""
                ^snmp-server\snotification-log-mib
                (\sdefault(?P<default>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server notification-log-mib default",
            "result": {
                "notification_log_mib": {
                    "default": "{{True if default is defined}}",
                },
            },
        },
        {
            "name": "notification_log_mib.disable",
            "getval": re.compile(
                r"""
                ^snmp-server\snotification-log-mib
                (\sdisable(?P<disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server notification-log-mib disable",
            "result": {
                "notification_log_mib": {
                    "disable": "{{True if disable is defined}}",
                },
            },
        },
        {
            "name": "notification_log_mib.GlobalSize",
            "getval": re.compile(
                r"""
                ^snmp-server\snotification-log-mib
                (\sGlobalSize\s(?P<gsize>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server notification-log-mib GlobalSize {{notification_log_mib.GlobalSize}}",
            "result": {
                "notification_log_mib": {
                    "GlobalSize": "{{gsize}}",

                },
            },
        },
        {
            "name": "oid_poll_stats",
            "getval": re.compile(
                r"""
                ^snmp-server(\soid-poll-stats(?P<oid_stats>))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server oid-poll-stats",
            "result": {
                "oid_poll_stats": "{{ True if oid_stats is defined }}",
            },
        },
        {
            "name": "overload_control",
            "getval": re.compile(
                r"""
                ^snmp-server\soverload-control
                (\s(?P<overload_drop_time>\d+))?
                (\s(?P<overload_throttle_rate>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": overload_control_tmplt,
            "result": {
                "overload_control": {
                    "overload_drop_time": "{{overload_drop_time}}",
                    "overload_throttle_rate": "{{overload_throttle_rate}}",
                },
            },
        },
        {
            "name": "packetsize",
            "getval": re.compile(
                r"""
                ^snmp-server(\spacketsize\s(?P<packetsize>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server packetsize {{packetsize}}",
            "result": {
                "packetsize": "{{ packetsize }}",
            },
        },
        {
            "name": "queue_length",
            "getval": re.compile(
                r"""
                ^snmp-server(\squeue-length\s(?P<queue_length>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server queue-length {{queue_length}}",
            "result": {
                "queue_length": "{{ queue_length }}",
            },
        },
        {
            "name": "targets",
            "getval": re.compile(
                r"""
                ^snmp-server(\starget\slist\s(?P<targets>\S+))
                (\shost\s(?P<host>\S+))?
                (\svrf\s(?P<vrf>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": targets_tmplt,
            "result": {
                "targets": [
                    {
                        "name": "{{ targets }}",
                        "host": "{{host}}",
                        "vrf": "{{vrf}}",
                    },
                ],
            },
        },
        {
            "name": "throttle_time",
            "getval": re.compile(
                r"""
                ^snmp-server(\sthrottle-time\s(?P<throttle_time>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server throttle-time {{throttle_time}}",
            "result": {
                "throttle_time": "{{ throttle_time }}",
            },
        },
        {
            "name": "timeouts.duplicate",
            "getval": re.compile(
                r"""
                ^snmp-server\stimeouts
                (\sduplicate\s(?P<duplicate>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server timeouts duplicate {{timeouts.duplicate}}",
            "result": {
                "timeouts": {
                    "duplicate": "{{duplicate}}",
                },
            },
        },
        {
            "name": "timeouts.inQdrop",
            "getval": re.compile(
                r"""
                ^snmp-server\stimeouts
                (\sinQdrop\s(?P<inQdrop>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server timeouts inQdrop {{timeouts.inQdrop}}",
            "result": {
                "timeouts": {
                    "inQdrop": "{{inQdrop}}",
                },
            },
        },
        {
            "name": "timeouts.subagent",
            "getval": re.compile(
                r"""
                ^snmp-server\stimeouts
                (\ssubagent\s(?P<subagent>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server timeouts subagent {{timeouts.subagent}}",
            "result": {
                "timeouts": {
                    "subagent": "{{subagent}}",

                },
            },
        },
        {
            "name": "timeouts.pdu_stats",
            "getval": re.compile(
                r"""
                ^snmp-server\stimeouts
                (\spdu\sstats\s(?P<pdu>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server timeouts pdu stats {{timeouts.pdu_stats}}",
            "result": {
                "timeouts": {
                    "pdu_stats": "{{pdu}}",

                },
            },
        },
        {
            "name": "timeouts.threshold",
            "getval": re.compile(
                r"""
                ^snmp-server\stimeouts
                (\sthreshold\s(?P<threshold>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server timeouts threshold {{timeouts.threshold}}",
            "result": {
                "timeouts": {
                    "threshold": "{{threshold}}",
                },
            },
        },
        {
            "name": "trap.throttle_time",
            "getval": re.compile(
                r"""
                ^snmp-server\strap
                (\sthrottle-time\s(?P<throttle_time>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server trap throttle-time {{trap.throttle_time}}",
            "result": {
                "trap": {
                    "throttle_time": "{{throttle_time}}",

                },
            },
        },
        {
            "name": "trap.authentication_vrf_disable",
            "getval": re.compile(
                r"""
                ^snmp-server\strap
                (\sauthentication\svrf\sdisable(?P<authentication_vrf_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server trap authentication vrf disable",
            "result": {
                "trap": {
                    "authentication_vrf_disable": "{{True if authentication_vrf_disable is defined}}",
                },
            },
        },
        {
            "name": "trap.link_ietf",
            "getval": re.compile(
                r"""
                ^snmp-server\strap
                (\slink\sietf(?P<link_ietf>))?
                (\sthrottle-time\s(?P<throttle_time>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server trap link ietf",
            "result": {
                "trap": {
                    "link_ietf": "{{True if link_ietf is defined}}",
                },
            },
        },
        {
            "name": "trap_source",
            "getval": re.compile(
                r"""
                ^snmp-server(\strap-source\s(?P<trap_source>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server trap-source {{trap_source}}",
            "result": {
                "trap_source": "{{ trap_source }}",
            },
        },
        {
            "name": "trap_timeout",
            "getval": re.compile(
                r"""
                ^snmp-server(\strap-timeout\s(?P<trap_timeout>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server trap-timeout {{trap_timeout}}",
            "result": {
                "trap_timeout": "{{ trap_timeout }}",
            },
        },
        {
            "name": "traps.addrpool.low",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\saddrpool\slow(?P<addrpool_low>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps addrpool low",
            "result": {
                "traps": {
                    "addrpool": {
                        "low": "{{True if addrpool_low is defined}}",
                    },
                },

            },
        },
        {
            "name": "traps.addrpool.high",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\saddrpool\shigh(?P<addrpool_high>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps addrpool high",
            "result": {
                "traps": {
                    "addrpool": {
                        "high": "{{True if addrpool_high is defined}}",
                    },
                },

            },
        },
        {
            "name": "traps.bfd",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sbfd(?P<bfd>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps bfd",
            "result": {
                "traps": {
                    "bfd": "{{True if bfd is defined}}",
                },
            },
        },
        {
            "name": "traps.bgp.cbgp2",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sbgp\scbgp2(?P<bgp_cgp2>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps bgp cbgp2",
            "result": {
                "traps": {

                    "bgp": {
                        "cbgp2": "{{True if bgp_cgp2 is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.bgp.updown",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sbgp\supdown(?P<bgp_updown>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps bgp updown",
            "result": {
                "traps": {
                    "bgp": {
                        "updown": "{{True if updown is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.bulkstat_collection",
            "getval": re.compile(
                r"""
                ^snmp-server\straps

                (\sbulkstat\scollection(?P<bulkstat_collection>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps bulkstat collection",
            "result": {
                "traps": {

                    "bulkstat_collection": "{{True if bulkstat_collection is defined}}",

                },
            },
        },
        {
            "name": "traps.bulkstat_transfer",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sbulkstat\stransfer(?P<bulkstat_t>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps bulkstat transfer",
            "result": {
                "traps": {

                    "bulkstat_transfer": "{{True if bulkstat_t is defined}}",

                },
            },
        },
        {
            "name": "traps.bridgemib",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sbridgemib(?P<bridgemib>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps bridgemib",
            "result": {
                "traps": {

                    "bridgemib": "{{True if bridgemib is defined}}",

                },
            },
        },
        {
            "name": "traps.copy_complete",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\scopy-complete(?P<copy_complete>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps copy-complete",
            "result": {
                "traps": {

                    "copy_complete": "{{True if copy_complete is defined}}",

                },
            },
        },
        {
            "name": "traps.cisco_entity_ext",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\scisco-entity-ext(?P<cee>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps cisco-entity-ext",
            "result": {
                "traps": {

                    "cisco_entity_ext": "{{True if cee is defined}}",

                },
            },
        },
        {
            "name": "traps.config",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sconfig(?P<config>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps config",
            "result": {
                "traps": {

                    "config": "{{True if config is defined}}",

                },
            },
        },
        {
            "name": "traps.diameter.peerdown",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sdiameter\speerdown(?P<peerdown>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps diameter peerdown",
            "result": {
                "traps": {
                    "diameter": {
                        "peerdown": "{{True if peerdown is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.diameter.peerup",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sdiameter\speerup(?P<peerup>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps diameter peerup",
            "result": {
                "traps": {
                    "diameter": {
                        "peerup": "{{True if peerup is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.diameter.protocolerror",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sdiameter\sprotocolerror(?P<protocolerror>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps diameter protocolerror",
            "result": {
                "traps": {
                    "diameter": {
                        "protocolerror": "{{True if protocolerror is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.diameter.permanentfail",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sdiameter\spermanentfail(?P<permanentfail>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps diameter permanentfail",
            "result": {
                "traps": {
                    "diameter": {
                        "permanentfail": "{{True if permanentfail is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.diameter.transientfail",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sdiameter\stransientfail(?P<transientfail>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps diameter transientfail",
            "result": {
                "traps": {
                    "diameter": {
                        "transientfail": "{{True if transientfail is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.entity",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sentity(?P<entity>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps entity",
            "result": {
                "traps": {
                    "entity": "{{True if entity is defined}}",
                },
            },
        },
        {
            "name": "traps.entity_redundancy.all",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sentity-redundancy\sall(?P<all>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps entity-redundancy all",
            "result": {
                "traps": {
                    "entity_redundancy": {
                        "all": "{{True if all is defined }}",

                    },

                },
            },
        },
        {
            "name": "traps.entity_redundancy.status",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sentity-redundancy\sstatus(?P<status>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps entity-redundancy status",
            "result": {
                "traps": {
                    "entity_redundancy": {
                        "status": "{{True if status is defined }}",

                    },

                },
            },
        },
        {
            "name": "traps.entity_redundancy.switchover",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sentity-redundancy\sswitchover(?P<switchover>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps entity-redundancy switchover",
            "result": {
                "traps": {
                    "entity_redundancy": {
                        "switchover": "{{True if switchover is defined }}",

                    },

                },
            },
        },
        {
            "name": "traps.entity_state.operstatus",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sentity-state\soperstatus(?P<operstatus>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps entity-state operstatus",
            "result": {
                "traps": {
                    "entity_state": {
                        "operstatus": "{{True if operstatus is defined }}",

                    },

                },
            },
        },
        {
            "name": "traps.entity_state.switchover",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sentity-state\sswitchover(?P<switchover>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps entity-state switchover",
            "result": {
                "traps": {
                    "entity_state": {
                        "switchover": "{{True if switchover is defined }}",

                    },

                },
            },
        },
        {
            "name": "traps.flash.insertion",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sflash\sinsertion(?P<f_insertion>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps flash insertion",
            "result": {
                "traps": {
                    "flash": {
                        "insertion": "{{True if f_insertion is defined }}",

                    },

                },
            },
        },
        {
            "name": "traps.flash.removal",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sflash\sremoval(?P<f_removal>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps flash removal",
            "result": {
                "traps": {
                    "flash": {
                        "removal": "{{True if f_removal is defined }}",

                    },

                },
            },
        },
        {
            "name": "traps.fru_ctrl",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sfru-ctrl(?P<fru_ctrl>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps fru-ctrl",
            "result": {
                "traps": {
                    "fru_ctrl": "{{True if fru_ctrl is defined }}",
                },
            },
        },
        {
            "name": "traps.hsrp",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\shsrp(?P<hsrp>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps hsrp",
            "result": {
                "traps": {
                    "hsrp": "{{True if hsrp is defined }}",
                },
            },
        },
        {
            "name": "traps.ipsla",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sipsla(?P<ipsla>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ipsla",
            "result": {
                "traps": {
                    "ipsla": "{{True if ipsla is defined }}",
                },
            },
        },
        {
            "name": "traps.ipsec.start",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sipsec\stunnel\sstart(?P<ipsec_start>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ipsec tunnel start",
            "result": {
                "traps": {
                    "ipsec": {
                        "start": "{{True if ipsec_start is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.ipsec.stop",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sipsec\stunnel\sstop(?P<ipsec_stop>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ipsec tunnel stop",
            "result": {
                "traps": {
                    "ipsec": {
                        "stop": "{{True if ipsec_stop is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.isakmp.start",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sisakmp\stunnel\sstart(?P<isakmp_start>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps isakmp tunnel start",
            "result": {
                "traps": {
                    "isakmp": {
                        "start": "{{True if isakmp_start is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.isakmp.stop",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\sisakmp\stunnel\sstop(?P<isakmp_stop>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps isakmp tunnel stop",
            "result": {
                "traps": {
                    "isakmp": {
                        "stop": "{{True if isakmp_stop is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.isis",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sisis\sall(?P<isis_all>))?
                (\sisis(\sdatabase-overload(?P<database_overload>))?(\smanual-address-drops(?P<manual_address_drops>))?
                (\scorrupted-lsp-detected(?P<corrupted_lsp_detected>))?
                (\sattempt-to-exceed-max-sequence(?P<attempt_to_exceed_max_sequence>))?
                (\sid-len-mismatch(?P<id_len_mismatch>))?
                (\smax-area-addresses-mismatch(?P<max_area_addresses_mismatch>))?
                (\sown-lsp-purge(?P<own_lsp_purge>))?
                (\ssequence-number-skip(?P<sequence_number_skip>))?
                (\sauthentication-type-failure(?P<authentication_type_failure>))?
                (\sauthentication-failure(?P<authentication_failure>))?
                (\sversion-skew(?P<version_skew>))?
                (\sarea-mismatch(?P<area_mismatch>))?
                (\srejected-adjacency(?P<rejected_adjacency>))?
                (\slsp-too-large-to-propagate(?P<lsp_too_large_to_propagate>))?
                (\sorig-lsp-buff-size-mismatch(?P<orig_lsp_buff_size_mismatch>))?
                (\sprotocols-supported-mismatch(?P<protocols_supported_mismatch>))?
                (\sadjacency-change(?P<adjacency_change>))?
                (\slsp-error-detected(?P<lsp_error_detected>))?)?
                $""", re.VERBOSE,
            ),
            "setval": tmplt_traps_isis,
            "result": {
                "traps": {

                    "isis": {
                        "all": "{{True if isis_all is defined}}",
                        "id_len_mismatch": "{{True if id_len_mismatch is defined}}",
                        "database_overload": "{{True if database_overload is defined}}",
                        "manual_address_drops": "{{True if manual_address_drops is defined}}",
                        "corrupted_lsp_detected": "{{True if corrupted_lsp_detected is defined}}",
                        "attempt_to_exceed_max_sequence": "{{True if attempt_to_exceed_max_sequence is defined}}",
                        "max_area_addresses_mismatch": "{{True if max_area_addresses_mismatch is defined}}",
                        "own_lsp_purge": "{{True if own_lsp_purge is defined}}",
                        "sequence_number_skip": "{{True if sequence_number_skip is defined}}",
                        "authentication_type_failure": "{{True if authentication_type_failure is defined}}",
                        "authentication_failure": "{{True if authentication_failure is defined}}",
                        "version_skew": "{{True if version_skew is defined}}",
                        "area_mismatch": "{{True if area_mismatch is defined}}",
                        "rejected_adjacency": "{{True if rejected_adjacency is defined}}",
                        "lsp_too_large_to_propagate": "{{True if lsp_too_large_to_propagate is defined}}",
                        "orig_lsp_buff_size_mismatch": "{{True if orig_lsp_buff_size_mismatch is defined}}",
                        "protocols_supported_mismatch": "{{True if protocols_supported_mismatch is defined}}",
                        "adjacency_change": "{{True if adjacency_change is defined}}",
                        "lsp_error_detected": "{{True if lsp_error_detected is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2tun.pseudowire_status",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2tun\spseudowire-status(?P<pseudowire_status>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2tun pseudowire-status",
            "result": {
                "traps": {

                    "l2tun": {
                        "pseudowire_status": "{{True if pseudowire_status is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2tun.sessions",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2tun\ssessions(?P<sessions>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2tun sessions",
            "result": {
                "traps": {

                    "l2tun": {
                        "sessions": "{{True if sessions is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2tun.tunnel_down",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2tun\stunnel-down(?P<tunnel_down>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2tun tunnel-down",
            "result": {
                "traps": {

                    "l2tun": {
                        "tunnel_down": "{{True if tunnel_down is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2tun.tunnel_up",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2tun\stunnel-up(?P<tunnel_up>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2tun tunnel-up",
            "result": {
                "traps": {

                    "l2tun": {
                        "tunnel_up": "{{True if tunnel_up is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2vpn.all",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2vpn\sall(?P<vpnall>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2vpn all",
            "result": {
                "traps": {

                    "l2vpn": {
                        "all": "{{True if vpnall is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2vpn.cisco",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2vpn\scisco(?P<cisco>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2vpn cisco",
            "result": {
                "traps": {

                    "l2vpn": {
                        "cisco": "{{True if cisco is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2vpn.vc_up",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2vpn\svc-up(?P<vc_up>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2vpn vc-up",
            "result": {
                "traps": {

                    "l2vpn": {
                        "vc_up": "{{True if vc_up is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.l2vpn.vc_down",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sl2vpn\svc-down(?P<vc_down>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps l2vpn vc-down",
            "result": {
                "traps": {

                    "l2vpn": {
                        "vc_down": "{{True if vc_down is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.msdp_peer_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                 (\smsdp\speer-state-change(?P<msdp>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps msdp peer-state-change",
            "result": {
                "traps": {

                    "msdp_peer_state_change": "{{True if msdp is defined }}",

                },
            },
        },
        {
            "name": "traps.ospf.retransmit.packets",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\sretransmit\spackets(?P<packets>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf retransmit packets",
            "result": {
                "traps": {

                    "ospf": {

                        "retransmit": {
                            "packets": "{{True if packets is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.retransmit.virt_packets",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\sretransmit\svirt-packets(?P<virt_packets>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf retransmit virt-packets",
            "result": {
                "traps": {

                    "ospf": {
                        "retransmit": {
                            "virt_packets": "{{True if virt_packets is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.lsa.lsa_maxage",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\slsa\slsa-maxage(?P<lsa_maxage>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf lsa lsa-maxage",
            "result": {
                "traps": {

                    "ospf": {

                        "lsa": {
                            "lsa_maxage": "{{True if lsa_maxage is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.lsa.lsa_originate",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\slsa\slsa-originate(?P<lsa_originate>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf lsa lsa-originate",
            "result": {
                "traps": {

                    "ospf": {

                        "lsa": {
                            "lsa_originate": "{{True if lsa_originate is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.errors.bad_packet",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\serrors\sbad-packet(?P<bad_packet>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf errors bad-packet",
            "result": {
                "traps": {

                    "ospf": {

                        "errors": {
                            "bad_packet": "{{True if bad_packet is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.errors.authentication_failure",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\serrors\sauthentication-failure(?P<authentication_failure_ospf>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf errors authentication-failure",
            "result": {
                "traps": {

                    "ospf": {

                        "errors": {
                            "authentication_failure": "{{True if authentication_failure_ospf is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.errors.config_error",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\serrors\sconfig-error(?P<config_error>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf errors config-error",
            "result": {
                "traps": {

                    "ospf": {

                        "errors": {
                            "config_error": "{{True if config_error is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.errors.virt_bad_packet",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\serrors\svirt-bad-packet(?P<virt_bad_packet>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf errors virt-bad-packet",
            "result": {
                "traps": {

                    "ospf": {

                        "errors": {
                            "virt_bad_packet": "{{True if virt_bad_packet is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.errors.virt_authentication_failure",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\serrors\svirt-authentication-failure(?P<virt_authentication_failure>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf errors virt-authentication-failure",
            "result": {
                "traps": {

                    "ospf": {

                        "errors": {
                            "virt_authentication_failure": "{{True if virt_authentication_failure is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.errors.virt_config_error",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\serrors\svirt-config-error(?P<virt_config_error>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf errors virt-config-error",
            "result": {
                "traps": {

                    "ospf": {

                        "errors": {
                            "virt_config_error": "{{True if virt_config_error is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.state_change.if_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\sstate-change\sif-state-change(?P<if_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf state-change if-state-change",
            "result": {
                "traps": {

                    "ospf": {

                        "state_change": {
                            "if_state_change": "{{True if if_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.state_change.neighbor_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\sstate-change\sneighbor-state-change(?P<neighbor_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf state-change neighbor-state-change",
            "result": {
                "traps": {

                    "ospf": {

                        "state_change": {
                            "neighbor_state_change": "{{True if neighbor_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.state_change.virtif_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\sstate-change\svirtif-state-change(?P<virtif_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf state-change virtif-state-change",
            "result": {
                "traps": {

                    "ospf": {

                        "state_change": {
                            "virtif_state_change": "{{True if virtif_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospf.state_change.virtneighbor_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospf\sstate-change\svirtneighbor-state-change(?P<virtneighbor_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospf state-change virtneighbor-state-change",
            "result": {
                "traps": {

                    "ospf": {

                        "state_change": {
                            "virtneighbor_state_change": "{{True if virtneighbor_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.errors.bad_packet",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\serrors\sbad-packet(?P<bad_packet>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 errors bad-packet",
            "result": {
                "traps": {

                    "ospfv3": {

                        "errors": {
                            "bad_packet": "{{True if bad_packet is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.errors.authentication_failure",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\serrors\sauthentication-failure(?P<authentication_failure_ospf>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 errors authentication-failure",
            "result": {
                "traps": {

                    "ospfv3": {

                        "errors": {
                            "authentication_failure": "{{True if authentication_failure_ospf is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.errors.config_error",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\serrors\sconfig-error(?P<config_error>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 errors config-error",
            "result": {
                "traps": {

                    "ospfv3": {

                        "errors": {
                            "config_error": "{{True if config_error is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.errors.virt_config_error",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\serrors\svirt-config-error(?P<virt_config_error>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 errors virt-config-error",
            "result": {
                "traps": {

                    "ospfv3": {

                        "errors": {
                            "virt_config_error": "{{True if virt_config_error is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.errors.virt_bad_packet",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\serrors\svirt-bad-packet(?P<virt_bad_packet>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 errors virt-bad-packet",
            "result": {
                "traps": {

                    "ospfv3": {

                        "errors": {
                            "virt_bad_packet": "{{True if virt_bad_packet is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.if_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\sif-state-change(?P<if_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change if-state-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "if_state_change": "{{True if if_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.neighbor_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\sneighbor-state-change(?P<neighbor_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change neighbor-state-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "neighbor_state_change": "{{True if neighbor_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.virtif_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\svirtif-state-change(?P<virtif_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change virtif-state-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "virtif_state_change": "{{True if virtif_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.virtneighbor_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\svirtneighbor-state-change(?P<virtneighbor_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change virtneighbor-state-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "virtneighbor_state_change": "{{True if virtneighbor_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.restart_status_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\srestart-status-change(?P<restart_status_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change restart-status-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "restart_status_change": "{{True if restart_status_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.restart_helper_status_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\srestart-helper-status-change(?P<restart_helper_status_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change restart-helper-status-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "restart_helper_status_change": "{{True if restart_helper_status_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.restart_virtual_helper_status_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\srestart-virtual-helper-status-change(?P<restart_virtual_helper_status_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change restart-virtual-helper-status-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "restart_virtual_helper_status_change": "{{True if restart_virtual_helper_status_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.ospfv3.state_change.nssa_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sospfv3\sstate-change\snssa-state-change(?P<nssa_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps ospfv3 state-change nssa-state-change",
            "result": {
                "traps": {

                    "ospfv3": {

                        "state_change": {
                            "nssa_state_change": "{{True if nssa_state_change is defined}}",

                        },

                    },

                },
            },
        },
        {
            "name": "traps.power",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\spower(?P<power>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps power",
            "result": {
                "traps": {
                    "power": "{{True if power is defined }}",

                },
            },
        },
        {
            "name": "traps.rf",
            "getval": re.compile(
                r"""
                ^snmp-server\straps


                (\spower(?P<power>))?
                (\srf(?P<rf>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps rf",
            "result": {
                "traps": {
                    "rf": "{{True if rf is defined}}",

                },
            },
        },
        {
            "name": "traps.pim.neighbor_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\spim\sneighbor-change(?P<neighbor_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps pim neighbor-change",
            "result": {
                "traps": {

                    "pim": {

                        "neighbor_change": "{{True if neighbor_change is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.pim.invalid_message_received",
            "getval": re.compile(
                r"""
                ^snmp-server\straps



                (\spim\sinvalid-message-received(?P<invalid_message_received>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps pim invalid-message-received",
            "result": {
                "traps": {

                    "pim": {

                        "invalid_message_received": "{{True if invalid_message_received is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.pim.rp_mapping_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps



                (\spim\srp-mapping-change(?P<rp_mapping_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps pim rp-mapping-change",
            "result": {
                "traps": {

                    "pim": {

                        "rp_mapping_change": "{{True if rp_mapping_change is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.pim.interface_state_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\spim\sinterface-state-change(?P<interface_state_change>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps pim interface-state-change",
            "result": {
                "traps": {
                    "pim": {
                        "interface_state_change": "{{True if interface_state_change is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.rsvp.lost_flow",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\srsvp\slost-flow(?P<lost_flow>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps rsvp lost-flow",
            "result": {
                "traps": {
                    "rsvp": {
                        "lost_flow": "{{True if lost_flow is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.rsvp.new_flow",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\srsvp\snew-flow(?P<new_flow>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps rsvp new-flow",
            "result": {
                "traps": {
                    "rsvp": {
                        "new_flow": "{{True if new_flow is defined}}",
                    },

                },

            },
        },
        {
            "name": "traps.rsvp.all",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\srsvp\sall(?P<all>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps rsvp all",
            "result": {
                "traps": {
                    "rsvp": {
                        "all": "{{True if all is defined}}",
                    },

                },

            },
        },
        {
            "name": "traps.selective_vrf_download_role_change",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\sselective-vrf-download\srole-change(?P<selective_vrf_download_role_change>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps selective-vrf-download role-change",
            "result": {
                "traps": {
                    "selective_vrf_download_role_change": "{{True if selective_vrf_download_role_change is defined}}",
                },
            },
        },
        {
            "name": "traps.sensor",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssensor(?P<sensor>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps sensor",
            "result": {
                "traps": {"sensor": "{{True if sensor is defined}}"},
            },
        },
        {
            "name": "traps.vrrp_events",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\svrrp\sevents(?P<vrrp_events>))?


                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps vrrp events",
            "result": {

                "traps": {
                    "vrrp_events": "{{True if vrrp_events is defined}}",
                },
            },
        },
        {
            "name": "traps.syslog",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssyslog(?P<syslog>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps syslog",
            "result": {
                "traps": {"syslog": "{{True if syslog is defined}}"},

            },
        },
        {
            "name": "traps.system",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssystem(?P<system>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps system",
            "result": {
                "traps": {"system": "{{True if system is defined}}"},

            },
        },
        {
            "name": "traps.subscriber.session_agg_access_interface",
            "getval": re.compile(
                r"""
                ^snmp-server\straps

                (\ssubscriber\ssession-agg\saccess-interface(?P<session_agg_access_interface>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps subscriber session-agg access-interface",
            "result": {
                "traps": {
                    "subscriber": {
                        "session_agg_access_interface": "{{True if session_agg_access_interface is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.subscriber.session_agg_node",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssubscriber\ssession-agg\snode(?P<session_agg_node>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps subscriber session-agg node",
            "result": {
                "traps": {
                    "subscriber": {
                        "session_agg_node": "{{True if session_agg_node is defined}}",
                    },

                },
            },

        },
        {
            "name": "traps.vpls.all",
            "getval": re.compile(
                r"""
                ^snmp-server\straps

                (\svpls\sall(?P<vpls_all>))?


                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps vpls all",
            "result": {
                "traps": {

                    "vpls": {
                        "all": "{{True if vpls_all is defined}}",

                    },

                },
            },
        },
        {
            "name": "traps.vpls.full_clear",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\svpls\sfull-clear(?P<full_clear>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps vpls full-clear",
            "result": {
                "traps": {
                    "vpls": {
                        "full_clear": "{{True if full_clear is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.vpls.full_raise",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\svpls\sfull-raise(?P<full_raise>))?

                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps vpls full-raise",
            "result": {
                "traps": {
                    "vpls": {
                        "full_raise": "{{True if full_raise is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.vpls.status",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\svpls\sstatus(?P<vpls_status>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps vpls status",
            "result": {
                "traps": {
                    "vpls": {
                        "status": "{{True if vpls_status is defined}}",
                    },

                },
            },
        },
        {
            "name": "traps.snmp.linkup",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssnmp\slinkup(?P<linkup>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps snmp linkup",
            "result": {
                "traps": {
                    "snmp": {
                        "linkup": "{{True if linkup is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.snmp.linkdown",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssnmp\slinkdown(?P<linkdown>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps snmp linkdown",
            "result": {
                "traps": {

                    "snmp": {
                        "linkdown": "{{True if linkdown is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.snmp.coldstart",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssnmp\scoldstart(?P<coldstart>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps snmp coldstart",
            "result": {
                "traps": {

                    "snmp": {

                        "coldstart": "{{True if coldstart is defined}}",

                    },
                },
            },
        },
        {
            "name": "traps.snmp.warmstart",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssnmp\swarmstart(?P<warmstart>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps snmp warmstart",
            "result": {
                "traps": {

                    "snmp": {
                        "warmstart": "{{True if warmstart is defined}}",
                    },
                },
            },
        },
        {
            "name": "traps.snmp.authentication",
            "getval": re.compile(
                r"""
                ^snmp-server\straps
                (\ssnmp\sauthentication(?P<authentication>))?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server traps snmp authentication",
            "result": {
                "traps": {

                    "snmp": {
                        "authentication": "{{True if authentication is defined}}",
                    },
                },
            },
        },
        {
            "name": "users",
            "getval": re.compile(
                r"""
                ^snmp-server\suser
                (\s(?P<name>\S+))
                (\s(?P<group>\S+))
                (\s(?P<version>v1|v2c|v3))
                (\sIPv4\s(?P<ipv4>\S+))?
                (\sIPv6\s(?P<ipv6>\S+))?
                (\s(?P<v4acl>\S+))?
                (\sSDROwner\s(?P<sdrowner>))?
                (\sSystemOwner\s(?P<systemowner>))?
                $""", re.VERBOSE,
            ),
            "setval": user_tmplt,
            "result": {
                "users": [
                    {
                        "user": "{{ name }}",
                        "group": "{{ group }}",
                        "acl_v4": "{{ipv4}}",
                        "acl_v6": "{{ipv6}}",
                        "SDROwner": "{{True if sdowner is defined}}",
                        "SystemOwner": "{{True if systemowner is defined }}",
                        "v4_acl": "{{v4acl}}",
                        "version": "{{version}}",
                    },
                ],
            },
        },
        {
            "name": "vrfs",
            "getval": re.compile(
                r"""
                ^snmp-server\svrf
                (\s(?P<vrf>\S+))
                (\scontext\s(?P<context>\S+))?
                ((\shost\s(?P<host>\S+))?
                (\s(?P<traps>traps))?
                (\s(?P<informs>informs))?
                (\sversion\s(?P<version>1|2c|3))?
                (\s(?P<community>\S+))?
                (\sudp-port\s(?P<port>\d+))?)?
                $""", re.VERBOSE,
            ),
            "setval": "snmp-server vrf {{vrf}}",
            "result": {
                "vrfs": {
                    "{{vrf}}": {
                        "vrf": "{{vrf}}",
                        "context": {
                            "name_{{context|d()}}": "{{context}}",
                        },
                        "hosts": [
                            {
                                "host": "{{ host }}",
                                "traps": "{{True if traps is defined}}",
                                "informs": "{{True if informs is defined}}",
                                "community": "{{community}}",
                                "udp_port": "{{port}}",
                                "version": "{{version}}",
                            },
                        ],
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: backup_schedule_config
short_description: Resource module for Backup Schedule Config
description:
- Manage operation create of the resource Backup Schedule Config.
- Schedules the configuration backup on the ISE node as per the input parameters. This API helps in creating the schedule for the first time.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  backupDescription:
    description: Description of the backup.
    type: str
  backupEncryptionKey:
    description: The encyption key for the backed up file. Encryption key must satisfy
      the following criteria - Contains at least one uppercase letter A-Z, Contains
      at least one lowercase letter a-z, Contains at least one digit 0-9, Contain only
      A-Za-z0-9_#, Has at least 8 characters, Has not more than 15 characters, Must
      not contain 'CcIiSsCco', Must not begin with.
    type: str
  backupName:
    description: The backup file will get saved with this name.
    type: str
  endDate:
    description: End date of the scheduled backup job. Allowed format MM/DD/YYYY. End
      date is not required in case of ONE_TIME frequency.
    type: str
  frequency:
    description: Backup Schedule Config's frequency.
    type: str
  monthDay:
    description: Day of month you want backup to be performed on when scheduled frequency
      is MONTHLY. Allowed values - from 1 to 28.
    type: str
  repositoryName:
    description: Name of the configured repository where the generated backup file will
      get copied.
    type: str
  startDate:
    description: Start date for scheduling the backup job. Allowed format MM/DD/YYYY.
    type: str
  status:
    description: Backup Schedule Config's status.
    type: str
  time:
    description: Time at which backup job get scheduled. Example- 12 00 AM.
    type: str
  weekDay:
    description: Backup Schedule Config's weekDay.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Backup And Restore
  description: Complete reference of the Backup And Restore API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!backup-and-restore-open-api
notes:
  - SDK Method used are
    backup_and_restore.BackupAndRestore.create_scheduled_config_backup,

  - Paths used are
    post /api/v1/backup-restore/config/schedule-config-backup,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.backup_schedule_config:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    backupDescription: string
    backupEncryptionKey: string
    backupName: string
    endDate: string
    frequency: string
    monthDay: string
    repositoryName: string
    startDate: string
    status: string
    time: string
    weekDay: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "message": "string",
        "link": {
          "rel": "string",
          "href": "string",
          "type": "string"
        }
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_eip
version_added: 5.0.0
short_description: manages EC2 elastic IP (EIP) addresses.
description:
  - This module can allocate or release an EIP.
  - This module can associate/disassociate an EIP with instances or network interfaces.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  device_id:
    description:
      - The id of the device for the EIP.
      - Can be an EC2 Instance id or Elastic Network Interface (ENI) id.
      - When specifying an ENI id, I(in_vpc) must be C(true)
      - The C(instance_id) alias was removed in release 6.0.0.
    required: false
    type: str
  public_ip:
    description:
      - The IP address of a previously allocated EIP.
      - When I(state=present) and device is specified, the EIP is associated with the device.
      - When I(state=absent) and device is specified, the EIP is disassociated from the device.
    aliases: [ ip ]
    type: str
  state:
    description:
      - When C(state=present), allocate an EIP or associate an existing EIP with a device.
      - When C(state=absent), disassociate the EIP from the device and optionally release it.
    choices: ['present', 'absent']
    default: present
    type: str
  in_vpc:
    description:
      - Allocate an EIP inside a VPC or not.
      - Required if specifying an ENI with I(device_id).
    default: false
    type: bool
  reuse_existing_ip_allowed:
    description:
      - Reuse an EIP that is not associated to a device (when available), instead of allocating a new one.
    default: false
    type: bool
  release_on_disassociation:
    description:
      - Whether or not to automatically release the EIP when it is disassociated.
    default: false
    type: bool
  private_ip_address:
    description:
      - The primary or secondary private IP address to associate with the Elastic IP address.
    type: str
  allow_reassociation:
    description:
      -  Specify this option to allow an Elastic IP address that is already associated with another
         network interface or instance to be re-associated with the specified instance or interface.
    default: false
    type: bool
  tag_name:
    description:
      - When I(reuse_existing_ip_allowed=true), supplement with this option to only reuse
        an Elastic IP if it is tagged with I(tag_name).
    type: str
  tag_value:
    description:
      - Supplements I(tag_name) but also checks that the value of the tag provided in I(tag_name) matches I(tag_value).
    type: str
  public_ipv4_pool:
    description:
      - Allocates the new Elastic IP from the provided public IPv4 pool (BYOIP)
        only applies to newly allocated Elastic IPs, isn't validated when I(reuse_existing_ip_allowed=true).
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3

author:
  - "Rick Mendes (@rickmendes) <rmendes@illumina.com>"
notes:
  - There may be a delay between the time the EIP is assigned and when
    the cloud instance is reachable via the new address. Use wait_for and
    pause to delay further playbook execution until the instance is reachable,
    if necessary.
  - This module returns multiple changed statuses on disassociation or release.
    It returns an overall status based on any changes occurring. It also returns
    individual changed statuses for disassociation and release.
  - Support for I(tags) and I(purge_tags) was added in release 2.1.0.
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: associate an elastic IP with an instance
  amazon.aws.ec2_eip:
    device_id: i-1212f003
    ip: 93.184.216.119

- name: associate an elastic IP with a device
  amazon.aws.ec2_eip:
    device_id: eni-c8ad70f3
    ip: 93.184.216.119

- name: associate an elastic IP with a device and allow reassociation
  amazon.aws.ec2_eip:
    device_id: eni-c8ad70f3
    public_ip: 93.184.216.119
    allow_reassociation: true

- name: disassociate an elastic IP from an instance
  amazon.aws.ec2_eip:
    device_id: i-1212f003
    ip: 93.184.216.119
    state: absent

- name: disassociate an elastic IP with a device
  amazon.aws.ec2_eip:
    device_id: eni-c8ad70f3
    ip: 93.184.216.119
    state: absent

- name: allocate a new elastic IP and associate it with an instance
  amazon.aws.ec2_eip:
    device_id: i-1212f003

- name: allocate a new elastic IP without associating it to anything
  amazon.aws.ec2_eip:
    state: present
  register: eip

- name: output the IP
  ansible.builtin.debug:
    msg: "Allocated IP is {{ eip.public_ip }}"

- name: provision new instances with ec2
  amazon.aws.ec2:
    keypair: mykey
    instance_type: c1.medium
    image: ami-40603AD1
    wait: true
    group: webserver
    count: 3
  register: ec2

- name: associate new elastic IPs with each of the instances
  amazon.aws.ec2_eip:
    device_id: "{{ item }}"
  loop: "{{ ec2.instance_ids }}"

- name: allocate a new elastic IP inside a VPC in us-west-2
  amazon.aws.ec2_eip:
    region: us-west-2
    in_vpc: true
  register: eip

- name: output the IP
  ansible.builtin.debug:
    msg: "Allocated IP inside a VPC is {{ eip.public_ip }}"

- name: allocate eip - reuse unallocated ips (if found) with FREE tag
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: FREE

- name: allocate eip - reuse unallocated ips if tag reserved is nope
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: reserved
    tag_value: nope

- name: allocate new eip - from servers given ipv4 pool
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    public_ipv4_pool: ipv4pool-ec2-0588c9b75a25d1a02

- name: allocate eip - from a given pool (if no free addresses where dev-servers tag is dynamic)
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: dev-servers
    public_ipv4_pool: ipv4pool-ec2-0588c9b75a25d1a02

- name: allocate eip from pool - check if tag reserved_for exists and value is our hostname
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: reserved_for
    tag_value: "{{ inventory_hostname }}"
    public_ipv4_pool: ipv4pool-ec2-0588c9b75a25d1a02
"""

RETURN = r"""
allocation_id:
  description: allocation_id of the elastic ip
  returned: on success
  type: str
  sample: eipalloc-51aa3a6c
public_ip:
  description: an elastic ip address
  returned: on success
  type: str
  sample: 52.88.159.209
"""

try:
    import botocore.exceptions
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


class EipError(Exception):
    pass


def associate_ip_and_device(
    ec2, module, address, private_ip_address, device_id, allow_reassociation, check_mode, is_instance=True
):
    if address_is_associated_with_device(ec2, module, address, device_id, is_instance):
        return {"changed": False}

    # If we're in check mode, nothing else to do
    if not check_mode:
        if is_instance:
            try:
                params = dict(
                    InstanceId=device_id,
                    AllowReassociation=allow_reassociation,
                )
                if private_ip_address:
                    params["PrivateIpAddress"] = private_ip_address
                if address["Domain"] == "vpc":
                    params["AllocationId"] = address["AllocationId"]
                else:
                    params["PublicIp"] = address["PublicIp"]
                res = ec2.associate_address(aws_retry=True, **params)
            except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                msg = f"Couldn't associate Elastic IP address with instance '{device_id}'"
                module.fail_json_aws(e, msg=msg)
        else:
            params = dict(
                NetworkInterfaceId=device_id,
                AllocationId=address["AllocationId"],
                AllowReassociation=allow_reassociation,
            )

            if private_ip_address:
                params["PrivateIpAddress"] = private_ip_address

            try:
                res = ec2.associate_address(aws_retry=True, **params)
            except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                msg = f"Couldn't associate Elastic IP address with network interface '{device_id}'"
                module.fail_json_aws(e, msg=msg)
        if not res:
            module.fail_json(msg="Association failed.")

    return {"changed": True}


def disassociate_ip_and_device(ec2, module, address, device_id, check_mode, is_instance=True):
    if not address_is_associated_with_device(ec2, module, address, device_id, is_instance):
        return {"changed": False}

    # If we're in check mode, nothing else to do
    if not check_mode:
        try:
            if address["Domain"] == "vpc":
                ec2.disassociate_address(AssociationId=address["AssociationId"], aws_retry=True)
            else:
                ec2.disassociate_address(PublicIp=address["PublicIp"], aws_retry=True)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            module.fail_json_aws(e, msg="Dissassociation of Elastic IP failed")

    return {"changed": True}


@AWSRetry.jittered_backoff()
def find_address(ec2, module, public_ip, device_id, is_instance=True):
    """Find an existing Elastic IP address"""
    filters = []
    kwargs = {}

    if public_ip:
        kwargs["PublicIps"] = [public_ip]
    elif device_id:
        if is_instance:
            filters.append({"Name": "instance-id", "Values": [device_id]})
        else:
            filters.append({"Name": "network-interface-id", "Values": [device_id]})

    if len(filters) > 0:
        kwargs["Filters"] = filters
    elif len(filters) == 0 and public_ip is None:
        return None

    try:
        addresses = ec2.describe_addresses(**kwargs)
    except is_boto3_error_code("InvalidAddress.NotFound") as e:
        # If we're releasing and we can't find it, it's already gone...
        if module.params.get("state") == "absent":
            module.exit_json(changed=False, disassociated=False, released=False)
        module.fail_json_aws(e, msg="Couldn't obtain list of existing Elastic IP addresses")

    addresses = addresses["Addresses"]
    if len(addresses) == 1:
        return addresses[0]
    elif len(addresses) > 1:
        msg = f"Found more than one address using args {kwargs} Addresses found: {addresses}"
        module.fail_json_aws(botocore.exceptions.ClientError, msg=msg)


def address_is_associated_with_device(ec2, module, address, device_id, is_instance=True):
    """Check if the elastic IP is currently associated with the device"""
    address = find_address(ec2, module, address["PublicIp"], device_id, is_instance)
    if address:
        if is_instance:
            if "InstanceId" in address and address["InstanceId"] == device_id:
                return address
        else:
            if "NetworkInterfaceId" in address and address["NetworkInterfaceId"] == device_id:
                return address
    return False


def allocate_address(
    ec2,
    module,
    domain,
    reuse_existing_ip_allowed,
    check_mode,
    tags,
    search_tags=None,
    public_ipv4_pool=None,
):
    """Allocate a new elastic IP address (when needed) and return it"""
    if not domain:
        domain = "standard"

    if reuse_existing_ip_allowed:
        filters = []
        filters.append({"Name": "domain", "Values": [domain]})

        if search_tags is not None:
            filters += ansible_dict_to_boto3_filter_list(search_tags)

        try:
            all_addresses = ec2.describe_addresses(Filters=filters, aws_retry=True)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            module.fail_json_aws(e, msg="Couldn't obtain list of existing Elastic IP addresses")

        all_addresses = all_addresses["Addresses"]

        if domain == "vpc":
            unassociated_addresses = [a for a in all_addresses if not a.get("AssociationId", None)]
        else:
            unassociated_addresses = [a for a in all_addresses if not a["InstanceId"]]
        if unassociated_addresses:
            return unassociated_addresses[0], False

    if public_ipv4_pool:
        return (
            allocate_address_from_pool(
                ec2,
                module,
                domain,
                check_mode,
                public_ipv4_pool,
                tags,
            ),
            True,
        )

    params = {"Domain": domain}
    if tags:
        params["TagSpecifications"] = boto3_tag_specifications(tags, types="elastic-ip")

    try:
        if check_mode:
            return None, True
        result = ec2.allocate_address(aws_retry=True, **params), True
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg="Couldn't allocate Elastic IP address")
    return result


def release_address(ec2, module, address, check_mode):
    """Release a previously allocated elastic IP address"""

    # If we're in check mode, nothing else to do
    if not check_mode:
        try:
            ec2.release_address(AllocationId=address["AllocationId"], aws_retry=True)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            module.fail_json_aws(e, msg="Couldn't release Elastic IP address")

    return {"changed": True}


@AWSRetry.jittered_backoff()
def describe_eni_with_backoff(ec2, module, device_id):
    try:
        return ec2.describe_network_interfaces(NetworkInterfaceIds=[device_id])
    except is_boto3_error_code("InvalidNetworkInterfaceID.NotFound") as e:
        module.fail_json_aws(e, msg="Couldn't get list of network interfaces.")


def find_device(ec2, module, device_id, is_instance=True):
    """Attempt to find the EC2 instance and return it"""

    if is_instance:
        try:
            paginator = ec2.get_paginator("describe_instances")
            reservations = list(paginator.paginate(InstanceIds=[device_id]).search("Reservations[]"))
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            module.fail_json_aws(e, msg="Couldn't get list of instances")

        if len(reservations) == 1:
            instances = reservations[0]["Instances"]
            if len(instances) == 1:
                return instances[0]
    else:
        try:
            interfaces = describe_eni_with_backoff(ec2, module, device_id)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            module.fail_json_aws(e, msg="Couldn't get list of network interfaces.")
        if len(interfaces) == 1:
            return interfaces[0]


def ensure_present(
    ec2,
    module,
    domain,
    address,
    private_ip_address,
    device_id,
    reuse_existing_ip_allowed,
    allow_reassociation,
    check_mode,
    tags,
    is_instance=True,
):
    changed = False

    # Return the EIP object since we've been given a public IP
    if not address:
        if check_mode:
            return {"changed": True}

        address, changed = allocate_address(
            ec2,
            module,
            domain,
            reuse_existing_ip_allowed,
            check_mode,
            tags,
        )

    if device_id:
        # Allocate an IP for instance since no public_ip was provided
        if is_instance:
            instance = find_device(ec2, module, device_id)
            if reuse_existing_ip_allowed:
                if instance["VpcId"] and len(instance["VpcId"]) > 0 and domain is None:
                    msg = "You must set 'in_vpc' to true to associate an instance with an existing ip in a vpc"
                    module.fail_json_aws(botocore.exceptions.ClientError, msg=msg)

            # Associate address object (provided or allocated) with instance
            assoc_result = associate_ip_and_device(
                ec2, module, address, private_ip_address, device_id, allow_reassociation, check_mode
            )
        else:
            instance = find_device(ec2, module, device_id, is_instance=False)
            # Associate address object (provided or allocated) with instance
            assoc_result = associate_ip_and_device(
                ec2, module, address, private_ip_address, device_id, allow_reassociation, check_mode, is_instance=False
            )

        changed = changed or assoc_result["changed"]

    return {"changed": changed, "public_ip": address["PublicIp"], "allocation_id": address["AllocationId"]}


def ensure_absent(ec2, module, address, device_id, check_mode, is_instance=True):
    if not address:
        return {"changed": False}

    # disassociating address from instance
    if device_id:
        if is_instance:
            return disassociate_ip_and_device(ec2, module, address, device_id, check_mode)
        else:
            return disassociate_ip_and_device(ec2, module, address, device_id, check_mode, is_instance=False)
    # releasing address
    else:
        return release_address(ec2, module, address, check_mode)


def allocate_address_from_pool(
    ec2,
    module,
    domain,
    check_mode,
    public_ipv4_pool,
    tags,
):
    # type: (EC2Connection, AnsibleAWSModule, str, bool, str) -> Address
    """Overrides botocore's allocate_address function to support BYOIP"""
    if check_mode:
        return None

    params = {}

    if domain is not None:
        params["Domain"] = domain

    if public_ipv4_pool is not None:
        params["PublicIpv4Pool"] = public_ipv4_pool

    if tags:
        params["TagSpecifications"] = boto3_tag_specifications(tags, types="elastic-ip")

    try:
        result = ec2.allocate_address(aws_retry=True, **params)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg="Couldn't allocate Elastic IP address")
    return result


def generate_tag_dict(module, tag_name, tag_value):
    # type: (AnsibleAWSModule, str, str) -> Optional[Dict]
    """Generates a dictionary to be passed as a filter to Amazon"""
    if tag_name and not tag_value:
        if tag_name.startswith("tag:"):
            tag_name = tag_name.strip("tag:")
        return {"tag-key": tag_name}

    elif tag_name and tag_value:
        if not tag_name.startswith("tag:"):
            tag_name = "tag:" + tag_name
        return {tag_name: tag_value}

    elif tag_value and not tag_name:
        module.fail_json(msg="parameters are required together: ('tag_name', 'tag_value')")


def check_is_instance(device_id, in_vpc):
    if not device_id:
        return False
    if device_id.startswith("i-"):
        return True

    if device_id.startswith("eni-") and not in_vpc:
        raise EipError("If you are specifying an ENI, in_vpc must be true")

    return False


def main():
    argument_spec = dict(
        device_id=dict(required=False),
        public_ip=dict(required=False, aliases=["ip"]),
        state=dict(required=False, default="present", choices=["present", "absent"]),
        in_vpc=dict(required=False, type="bool", default=False),
        reuse_existing_ip_allowed=dict(required=False, type="bool", default=False),
        release_on_disassociation=dict(required=False, type="bool", default=False),
        allow_reassociation=dict(type="bool", default=False),
        private_ip_address=dict(),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(required=False, type="bool", default=True),
        tag_name=dict(),
        tag_value=dict(),
        public_ipv4_pool=dict(),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_by={
            "private_ip_address": ["device_id"],
        },
    )

    ec2 = module.client("ec2", retry_decorator=AWSRetry.jittered_backoff())

    device_id = module.params.get("device_id")
    public_ip = module.params.get("public_ip")
    private_ip_address = module.params.get("private_ip_address")
    state = module.params.get("state")
    in_vpc = module.params.get("in_vpc")
    domain = "vpc" if in_vpc else None
    reuse_existing_ip_allowed = module.params.get("reuse_existing_ip_allowed")
    release_on_disassociation = module.params.get("release_on_disassociation")
    allow_reassociation = module.params.get("allow_reassociation")
    tag_name = module.params.get("tag_name")
    tag_value = module.params.get("tag_value")
    public_ipv4_pool = module.params.get("public_ipv4_pool")
    tags = module.params.get("tags")
    purge_tags = module.params.get("purge_tags")

    try:
        is_instance = check_is_instance(device_id, in_vpc)
    except EipError as e:
        module.fail_json(msg=str(e))

    # Tags for *searching* for an EIP.
    search_tags = generate_tag_dict(module, tag_name, tag_value)

    try:
        if device_id:
            address = find_address(ec2, module, public_ip, device_id, is_instance=is_instance)
        else:
            address = find_address(ec2, module, public_ip, None)

        if state == "present":
            if device_id:
                result = ensure_present(
                    ec2,
                    module,
                    domain,
                    address,
                    private_ip_address,
                    device_id,
                    reuse_existing_ip_allowed,
                    allow_reassociation,
                    module.check_mode,
                    tags,
                    is_instance=is_instance,
                )
                if "allocation_id" not in result:
                    # Don't check tags on check_mode here - no EIP to pass through
                    module.exit_json(**result)
            else:
                if address:
                    result = {
                        "changed": False,
                        "public_ip": address["PublicIp"],
                        "allocation_id": address["AllocationId"],
                    }
                else:
                    address, changed = allocate_address(
                        ec2,
                        module,
                        domain,
                        reuse_existing_ip_allowed,
                        module.check_mode,
                        tags,
                        search_tags,
                        public_ipv4_pool,
                    )
                    if address:
                        result = {
                            "changed": changed,
                            "public_ip": address["PublicIp"],
                            "allocation_id": address["AllocationId"],
                        }
                    else:
                        # Don't check tags on check_mode here - no EIP to pass through
                        result = {"changed": changed}
                        module.exit_json(**result)

            result["changed"] |= ensure_ec2_tags(
                ec2, module, result["allocation_id"], resource_type="elastic-ip", tags=tags, purge_tags=purge_tags
            )
        else:
            if device_id:
                disassociated = ensure_absent(
                    ec2, module, address, device_id, module.check_mode, is_instance=is_instance
                )

                if release_on_disassociation and disassociated["changed"]:
                    released = release_address(ec2, module, address, module.check_mode)
                    result = {
                        "changed": True,
                        "disassociated": disassociated["changed"],
                        "released": released["changed"],
                    }
                else:
                    result = {
                        "changed": disassociated["changed"],
                        "disassociated": disassociated["changed"],
                        "released": False,
                    }
            else:
                released = release_address(ec2, module, address, module.check_mode)
                result = {"changed": released["changed"], "disassociated": False, "released": released["changed"]}

    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(str(e))

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
from ansible.plugins.action import ActionBase

try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.ise.plugins.plugin_utils.ise import (
    ISESDK,
    ise_argument_spec,
    ise_compare_equality,
    get_dict_result,
)

# Get common arguments specification
argument_spec = ise_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present", "absent"]),
    name=dict(type="str"),
    description=dict(type="str"),
    ndgtype=dict(type="str"),
    id=dict(type="str"),
))

required_if = [
    ("state", "present", ["id", "name"], True),
    ("state", "absent", ["id", "name"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class NetworkDeviceGroup(object):
    def __init__(self, params, ise):
        self.ise = ise
        self.new_object = dict(
            name=params.get("name"),
            description=params.get("description"),
            ndgtype=params.get("ndgtype"),
            id=params.get("id"),
        )

    def get_object_by_name(self, name):
        query_name = name
        if query_name:
            query_name = query_name.replace('#', ':')
        try:
            result = self.ise.exec(
                family="network_device_group",
                function="get_network_device_group_by_name",
                params={"name": query_name},
                handle_func_exception=False,
            ).response['NetworkDeviceGroup']
            result = get_dict_result(result, 'name', name)
        except (TypeError, AttributeError) as e:
            self.ise.fail_json(
                msg=(
                    "An error occured when executing operation."
                    " Check the configuration of your API Settings and API Gateway settings on your ISE server."
                    " This collection assumes that the API Gateway, the ERS APIs and OpenAPIs are enabled."
                    " You may want to enable the (ise_debug: True) argument."
                    " The error was: {error}"
                ).format(error=e)
            )
        except Exception:
            result = None
        return result

    def get_object_by_id(self, id):
        try:
            result = self.ise.exec(
                family="network_device_group",
                function="get_network_device_group_by_id",
                handle_func_exception=False,
                params={"id": id}
            ).response['NetworkDeviceGroup']
        except (TypeError, AttributeError) as e:
            self.ise.fail_json(
                msg=(
                    "An error occured when executing operation."
                    " Check the configuration of your API Settings and API Gateway settings on your ISE server."
                    " This collection assumes that the API Gateway, the ERS APIs and OpenAPIs are enabled."
                    " You may want to enable the (ise_debug: True) argument."
                    " The error was: {error}"
                ).format(error=e)
            )
        except Exception:
            result = None
        return result

    def exists(self):
        result = False
        prev_obj = None
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        if id:
            prev_obj = self.get_object_by_id(id)
            result = prev_obj is not None and isinstance(prev_obj, dict)
        elif name:
            prev_obj = self.get_object_by_name(name)
            result = prev_obj is not None and isinstance(prev_obj, dict)
        return (result, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("name", "name"),
            ("description", "description"),
            ("ndgtype", "ndgtype"),
            ("id", "id"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        return any(not ise_compare_equality(current_obj.get(ise_param),
                                            requested_obj.get(ansible_param))
                   for (ise_param, ansible_param) in obj_params)

    def create(self):
        result = self.ise.exec(
            family="network_device_group",
            function="create_network_device_group",
            params=self.new_object,
        ).response
        return result

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        if not id:
            id_ = self.get_object_by_name(name).get("id")
            self.new_object.update(dict(id=id_))
        result = self.ise.exec(
            family="network_device_group",
            function="update_network_device_group_by_id",
            params=self.new_object
        ).response
        return result

    def delete(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        if not id:
            id_ = self.get_object_by_name(name).get("id")
            self.new_object.update(dict(id=id_))
        result = self.ise.exec(
            family="network_device_group",
            function="delete_network_device_group_by_id",
            params=self.new_object
        ).response
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        ise = ISESDK(params=self._task.args)
        obj = NetworkDeviceGroup(self._task.args, ise)

        state = self._task.args.get("state")

        response = None

        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    ise_update_response = obj.update()
                    self._result.update(dict(ise_update_response=ise_update_response))
                    (obj_exists, updated_obj) = obj.exists()
                    response = updated_obj
                    has_changed = None
                    has_changed = ise_update_response.get("UpdatedFieldsList").get("updatedField")
                    if (len(has_changed) == 0 or
                       has_changed[0].get("newValue") == "" and
                       has_changed[0].get("newValue") == has_changed[0].get("oldValue")):
                        self._result.pop("ise_update_response", None)
                        ise.object_already_present()
                    else:
                        ise.object_updated()
                else:
                    response = prev_obj
                    ise.object_already_present()
            else:
                ise_create_response = obj.create()
                (obj_exists, created_obj) = obj.exists()
                response = created_obj
                ise.object_created()

        elif state == "absent":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                obj.delete()
                response = prev_obj
                ise.object_deleted()
            else:
                ise.object_already_absent()

        self._result.update(dict(ise_response=response))
        self._result.update(ise.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_eip_info
version_added: 5.0.0
short_description: List EC2 EIP details
description:
  - List details of EC2 Elastic IP addresses.
  - This module was originally added to C(community.aws) in release 1.0.0.
author:
  - "Brad Macpherson (@iiibrad)"
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and filter
        value.  See U(https://docs.aws.amazon.com/cli/latest/reference/ec2/describe-addresses.html#options)
        for possible filters. Filter names and values are case sensitive.
    required: false
    default: {}
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details or the AWS region,
# see the AWS Guide for details.

- name: List all EIP addresses in the current region.
  amazon.aws.ec2_eip_info:
  register: regional_eip_addresses

- name: List all EIP addresses for a VM.
  amazon.aws.ec2_eip_info:
    filters:
      instance-id: i-123456789
  register: my_vm_eips

- ansible.builtin.debug:
    msg: "{{ my_vm_eips.addresses | selectattr('private_ip_address', 'equalto', '10.0.0.5') }}"

- name: List all EIP addresses for several VMs.
  amazon.aws.ec2_eip_info:
    filters:
      instance-id:
        - i-123456789
        - i-987654321
  register: my_vms_eips

- name: List all EIP addresses using the 'Name' tag as a filter.
  amazon.aws.ec2_eip_info:
    filters:
      tag:Name: www.example.com
  register: my_vms_eips

- name: List all EIP addresses using the Allocation-id as a filter
  amazon.aws.ec2_eip_info:
    filters:
      allocation-id: eipalloc-64de1b01
  register: my_vms_eips

# Set the variable eip_alloc to the value of the first allocation_id
# and set the variable my_pub_ip to the value of the first public_ip
- ansible.builtin.set_fact:
    eip_alloc: my_vms_eips.addresses[0].allocation_id
    my_pub_ip: my_vms_eips.addresses[0].public_ip
"""


RETURN = r"""
addresses:
  description: Properties of all Elastic IP addresses matching the provided filters. Each element is a dict with all the information related to an EIP.
  returned: on success
  type: list
  sample: [{
        "allocation_id": "eipalloc-64de1b01",
        "association_id": "eipassoc-0fe9ce90d6e983e97",
        "domain": "vpc",
        "instance_id": "i-01020cfeb25b0c84f",
        "network_interface_id": "eni-02fdeadfd4beef9323b",
        "network_interface_owner_id": "0123456789",
        "private_ip_address": "10.0.0.1",
        "public_ip": "54.81.104.1",
        "tags": {
            "Name": "test-vm-54.81.104.1"
        }
    }]
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by imported AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def get_eips_details(module):
    connection = module.client("ec2", retry_decorator=AWSRetry.jittered_backoff())
    filters = module.params.get("filters")
    try:
        response = connection.describe_addresses(aws_retry=True, Filters=ansible_dict_to_boto3_filter_list(filters))
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Error retrieving EIPs")

    addresses = camel_dict_to_snake_dict(response)["addresses"]
    for address in addresses:
        if "tags" in address:
            address["tags"] = boto3_tag_list_to_ansible_dict(address["tags"])
    return addresses


def main():
    module = AnsibleAWSModule(argument_spec=dict(filters=dict(type="dict", default={})), supports_check_mode=True)

    module.exit_json(changed=False, addresses=get_eips_details(module))


if __name__ == "__main__":
    main()



# Copyright: (c) 2021, Aubin Bikouo (@abikouo)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
name: turbo_demo
author:
  - Aubin Bikouo (@abikouo)

short_description: A demo for lookup plugins on cloud.common
description:
  - return the parent process of the running process
options:
  playbook_vars:
    description: list of playbook variables to add in the output.
    type: list
    elements: str
"""

EXAMPLES = r"""
"""

RETURN = r"""
"""


import os
import sys
import traceback

from ansible_collections.cloud.common.plugins.plugin_utils.turbo.lookup import (
    TurboLookupBase as LookupBase,
)


def counter():
    counter.i += 1
    return counter.i


# NOTE: workaround to avoid a warning with ansible-doc
if True:  # pylint: disable=using-constant-test
    counter.i = 0


async def execute(terms, variables, playbook_vars):
    result = []
    result.append("running from pid: {pid}".format(pid=os.getpid()))
    if playbook_vars is not None:
        result += [
            variables["vars"].get(x) for x in playbook_vars if x in variables["vars"]
        ]
    if terms:
        result += terms

    for id, stack in list(sys._current_frames().items()):
        for fname, line_id, name, line in traceback.extract_stack(stack):
            if fname == __file__:
                continue

    result.append("turbo_demo_counter: {0}".format(counter()))
    return result


class LookupModule(LookupBase):
    async def _run(self, terms, variables=None, playbook_vars=None):
        result = await execute(terms, variables, playbook_vars)
        return result

    run = _run if not hasattr(LookupBase, "run_on_daemon") else LookupBase.run_on_daemon



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: swim_image_details_info
short_description: Information module for Swim Image Details
description:
- Get all Swim Image Details.
- Returns software image list based on a filter criteria. For example "filterbyName = cat3k%".
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  imageUuid:
    description:
    - ImageUuid query parameter.
    type: str
  name:
    description:
    - Name query parameter.
    type: str
  family:
    description:
    - Family query parameter.
    type: str
  applicationType:
    description:
    - ApplicationType query parameter.
    type: str
  imageIntegrityStatus:
    description:
    - ImageIntegrityStatus query parameter. ImageIntegrityStatus - FAILURE, UNKNOWN, VERIFIED.
    type: str
  version:
    description:
    - Version query parameter. Software Image Version.
    type: str
  imageSeries:
    description:
    - ImageSeries query parameter. Image Series.
    type: str
  imageName:
    description:
    - ImageName query parameter. Image Name.
    type: str
  isTaggedGolden:
    description:
    - IsTaggedGolden query parameter. Is Tagged Golden.
    type: bool
  isCCORecommended:
    description:
    - IsCCORecommended query parameter. Is recommended from cisco.com.
    type: bool
  isCCOLatest:
    description:
    - IsCCOLatest query parameter. Is latest from cisco.com.
    type: bool
  createdTime:
    description:
    - CreatedTime query parameter. Time in milliseconds (epoch format).
    type: int
  imageSizeGreaterThan:
    description:
    - ImageSizeGreaterThan query parameter. Size in bytes.
    type: int
  imageSizeLesserThan:
    description:
    - ImageSizeLesserThan query parameter. Size in bytes.
    type: int
  sortBy:
    description:
    - SortBy query parameter. Sort results by this field.
    type: str
  sortOrder:
    description:
    - SortOrder query parameter. Sort order - 'asc' or 'des'. Default is asc.
    type: str
  limit:
    description:
    - Limit query parameter.
    type: int
  offset:
    description:
    - Offset query parameter.
    type: int
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) GetSoftwareImageDetails
  description: Complete reference of the GetSoftwareImageDetails API.
  link: https://developer.cisco.com/docs/dna-center/#!get-software-image-details
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.get_software_image_details,

  - Paths used are
    get /dna/intent/api/v1/image/importation,

"""

EXAMPLES = r"""
- name: Get all Swim Image Details
  cisco.dnac.swim_image_details_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    imageUuid: string
    name: string
    family: string
    applicationType: string
    imageIntegrityStatus: string
    version: string
    imageSeries: string
    imageName: string
    isTaggedGolden: True
    isCCORecommended: True
    isCCOLatest: True
    createdTime: 0
    imageSizeGreaterThan: 0
    imageSizeLesserThan: 0
    sortBy: string
    sortOrder: string
    limit: 0
    offset: 0
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "applicableDevicesForImage": [
            {
              "mdfId": "string",
              "productId": [
                "string"
              ],
              "productName": "string"
            }
          ],
          "applicationType": "string",
          "createdTime": "string",
          "extendedAttributes": {},
          "family": "string",
          "feature": "string",
          "fileServiceId": "string",
          "fileSize": "string",
          "imageIntegrityStatus": "string",
          "imageName": "string",
          "imageSeries": [
            "string"
          ],
          "imageSource": "string",
          "imageType": "string",
          "imageUuid": "string",
          "importSourceType": "string",
          "isTaggedGolden": true,
          "md5Checksum": "string",
          "name": "string",
          "profileInfo": [
            {
              "description": "string",
              "extendedAttributes": {},
              "memory": 0,
              "productType": "string",
              "profileName": "string",
              "shares": 0,
              "vCpu": 0
            }
          ],
          "shaCheckSum": "string",
          "vendor": "string",
          "version": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_admins
short_description: Resource module for organizations _admins
description:
- Manage operations create, update and delete of the resource organizations _admins.
- Create a new dashboard administrator.
- Revoke all access for a dashboard administrator within this organization.
- Update an administrator.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  adminId:
    description: AdminId path parameter. Admin ID.
    type: str
  authenticationMethod:
    description: The method of authentication the user will use to sign in to the Meraki
      dashboard. Can be one of 'Email' or 'Cisco SecureX Sign-On'. The default is Email
      authentication.
    type: str
  email:
    description: The email of the dashboard administrator. This attribute can not be
      updated.
    type: str
  name:
    description: The name of the dashboard administrator.
    type: str
  networks:
    description: The list of networks that the dashboard administrator has privileges
      on.
    elements: dict
    suboptions:
      access:
        description: The privilege of the dashboard administrator on the network. Can
          be one of 'full', 'read-only', 'guest-ambassador' or 'monitor-only'.
        type: str
      id:
        description: The network ID.
        type: str
    type: list
  orgAccess:
    description: The privilege of the dashboard administrator on the organization. Can
      be one of 'full', 'read-only', 'enterprise' or 'none'.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  tags:
    description: The list of tags that the dashboard administrator has privileges on.
    elements: dict
    suboptions:
      access:
        description: The privilege of the dashboard administrator on the tag. Can be
          one of 'full', 'read-only', 'guest-ambassador' or 'monitor-only'.
        type: str
      tag:
        description: The name of the tag.
        type: str
    type: list
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for organizations createOrganizationAdmin
  description: Complete reference of the createOrganizationAdmin API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-organization-admin
- name: Cisco Meraki documentation for organizations deleteOrganizationAdmin
  description: Complete reference of the deleteOrganizationAdmin API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-admin
- name: Cisco Meraki documentation for organizations updateOrganizationAdmin
  description: Complete reference of the updateOrganizationAdmin API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-organization-admin
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_admin,
    organizations.Organizations.delete_organization_admin,
    organizations.Organizations.update_organization_admin,

  - Paths used are
    post /organizations/{organizationId}/admins,
    delete /organizations/{organizationId}/admins/{adminId},
    put /organizations/{organizationId}/admins/{adminId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_admins:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    authenticationMethod: Email
    email: miles@meraki.com
    name: Miles Meraki
    networks:
    - access: full
      id: N_24329156
    orgAccess: none
    organizationId: string
    tags:
    - access: read-only
      tag: west

- name: Update by id
  cisco.meraki.organizations_admins:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    adminId: string
    name: Miles Meraki
    networks:
    - access: full
      id: N_24329156
    orgAccess: none
    organizationId: string
    tags:
    - access: read-only
      tag: west

- name: Delete by id
  cisco.meraki.organizations_admins:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    adminId: string
    organizationId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "email": "string",
      "orgAccess": "string",
      "accountStatus": "string",
      "twoFactorAuthEnabled": true,
      "hasApiKey": true,
      "lastActive": "string",
      "tags": [
        {
          "tag": "string",
          "access": "string"
        }
      ],
      "networks": [
        {
          "id": "string",
          "access": "string"
        }
      ],
      "authenticationMethod": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: rds_subnet_group
version_added: 5.0.0
short_description: manage RDS database subnet groups
description:
  - Creates, modifies, and deletes RDS database subnet groups.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  state:
    description:
      - Specifies whether the subnet should be present or absent.
    required: true
    choices: [ 'present' , 'absent' ]
    type: str
  name:
    description:
      - Database subnet group identifier.
    required: true
    type: str
  description:
    description:
      - Database subnet group description.
      - Required when I(state=present).
    type: str
  subnets:
    description:
      - List of subnet IDs that make up the database subnet group.
      - Required when I(state=present).
    type: list
    elements: str
notes:
  - Support for I(tags) and I(purge_tags) was added in release 3.2.0.
author:
  - "Scott Anderson (@tastychutney)"
  - "Alina Buzachis (@alinabuzachis)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Add or change a subnet group
  amazon.aws.rds_subnet_group:
    state: present
    name: norwegian-blue
    description: My Fancy Ex Parrot Subnet Group
    subnets:
      - subnet-aaaaaaaa
      - subnet-bbbbbbbb

- name: Add or change a subnet group and associate tags
  amazon.aws.rds_subnet_group:
    state: present
    name: norwegian-blue
    description: My Fancy Ex Parrot Subnet Group
    subnets:
      - subnet-aaaaaaaa
      - subnet-bbbbbbbb
    tags:
      tag1: Tag1
      tag2: Tag2

- name: Remove a subnet group
  amazon.aws.rds_subnet_group:
    state: absent
    name: norwegian-blue
"""

RETURN = r"""
changed:
    description: True if listing the RDS subnet group succeeds.
    type: bool
    returned: always
    sample: "false"
subnet_group:
    description: Dictionary of DB subnet group values
    returned: I(state=present)
    type: complex
    contains:
        name:
            description: The name of the DB subnet group (maintained for backward compatibility)
            returned: I(state=present)
            type: str
            sample: "ansible-test-mbp-13950442"
        db_subnet_group_name:
            description: The name of the DB subnet group
            returned: I(state=present)
            type: str
            sample: "ansible-test-mbp-13950442"
        description:
            description: The description of the DB subnet group (maintained for backward compatibility)
            returned: I(state=present)
            type: str
            sample: "Simple description."
        db_subnet_group_description:
            description: The description of the DB subnet group
            returned: I(state=present)
            type: str
            sample: "Simple description."
        vpc_id:
            description: The VpcId of the DB subnet group
            returned: I(state=present)
            type: str
            sample: "vpc-0acb0ba033ff2119c"
        subnet_ids:
            description: Contains a list of Subnet IDs
            returned: I(state=present)
            type: list
            sample:
                "subnet-08c94870f4480797e"
        subnets:
            description: Contains a list of Subnet elements (@see https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/rds.html#RDS.Client.describe_db_subnet_groups) # noqa
            returned: I(state=present)
            type: list
            contains:
                subnet_availability_zone:
                    description: Contains Availability Zone information.
                    returned: I(state=present)
                    type: dict
                    version_added: 3.2.0
                    version_added_collection: community.aws
                    sample:
                        name: "eu-north-1b"
                subnet_identifier:
                    description: The identifier of the subnet.
                    returned: I(state=present)
                    type: str
                    version_added: 3.2.0
                    version_added_collection: community.aws
                    sample: "subnet-08c94870f4480797e"
                subnet_outpost:
                    description: This value specifies the Outpost.
                    returned: I(state=present)
                    type: dict
                    version_added: 3.2.0
                    version_added_collection: community.aws
                    sample: {}
                subnet_status:
                    description: The status of the subnet.
                    returned: I(state=present)
                    type: str
                    version_added: 3.2.0
                    version_added_collection: community.aws
                    sample: "Active"
        status:
            description: The status of the DB subnet group (maintained for backward compatibility)
            returned: I(state=present)
            type: str
            sample: "Complete"
        subnet_group_status:
            description: The status of the DB subnet group
            returned: I(state=present)
            type: str
            sample: "Complete"
        db_subnet_group_arn:
            description: The ARN of the DB subnet group
            returned: I(state=present)
            type: str
            sample: "arn:aws:rds:eu-north-1:123456789012:subgrp:ansible-test-13950442"
        tags:
            description: The tags associated with the subnet group
            returned: I(state=present)
            type: dict
            version_added: 3.2.0
            version_added_collection: community.aws
            sample:
                tag1: Tag1
                tag2: Tag2
"""

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import ensure_tags
from ansible_collections.amazon.aws.plugins.module_utils.rds import get_tags
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule


def create_result(changed, subnet_group=None):
    if subnet_group is None:
        return dict(changed=changed)
    result_subnet_group = dict(subnet_group)
    result_subnet_group["name"] = result_subnet_group.get("db_subnet_group_name")
    result_subnet_group["description"] = result_subnet_group.get("db_subnet_group_description")
    result_subnet_group["status"] = result_subnet_group.get("subnet_group_status")
    result_subnet_group["subnet_ids"] = create_subnet_list(subnet_group.get("subnets"))
    return dict(changed=changed, subnet_group=result_subnet_group)


@AWSRetry.jittered_backoff()
def _describe_db_subnet_groups_with_backoff(client, **kwargs):
    paginator = client.get_paginator("describe_db_subnet_groups")
    return paginator.paginate(**kwargs).build_full_result()


def get_subnet_group(client, module):
    params = dict()
    params["DBSubnetGroupName"] = module.params.get("name").lower()

    try:
        _result = _describe_db_subnet_groups_with_backoff(client, **params)
    except is_boto3_error_code("DBSubnetGroupNotFoundFault"):
        return None
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Couldn't describe subnet groups.")

    if _result:
        result = camel_dict_to_snake_dict(_result["DBSubnetGroups"][0])
        result["tags"] = get_tags(client, module, result["db_subnet_group_arn"])

    return result


def create_subnet_list(subnets):
    r"""
    Construct a list of subnet ids from a list of subnets dicts returned by boto3.
    Parameters:
        subnets (list): A list of subnets definitions.
        @see https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/rds.html#RDS.Client.describe_db_subnet_groups
    Returns:
        (list): List of subnet ids (str)
    """
    subnets_ids = []
    for subnet in subnets:
        subnets_ids.append(subnet.get("subnet_identifier"))
    return subnets_ids


def main():
    argument_spec = dict(
        state=dict(required=True, choices=["present", "absent"]),
        name=dict(required=True),
        description=dict(required=False),
        subnets=dict(required=False, type="list", elements="str"),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
    )
    required_if = [("state", "present", ["description", "subnets"])]

    module = AnsibleAWSModule(argument_spec=argument_spec, required_if=required_if, supports_check_mode=True)

    state = module.params.get("state")
    group_name = module.params.get("name").lower()
    group_description = module.params.get("description")
    group_subnets = module.params.get("subnets") or []

    try:
        connection = module.client("rds", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, "Failed to instantiate AWS connection.")

    # Default.
    changed = None
    result = create_result(False)
    tags_update = False
    subnet_update = False

    if module.params.get("tags") is not None:
        _tags = ansible_dict_to_boto3_tag_list(module.params.get("tags"))
    else:
        _tags = list()

    matching_groups = get_subnet_group(connection, module)

    if state == "present":
        if matching_groups:
            # We have one or more subnets at this point.

            # Check if there is any tags update
            tags_update = ensure_tags(
                connection,
                module,
                matching_groups["db_subnet_group_arn"],
                matching_groups["tags"],
                module.params.get("tags"),
                module.params["purge_tags"],
            )

            # Sort the subnet groups before we compare them
            existing_subnets = create_subnet_list(matching_groups["subnets"])
            existing_subnets.sort()
            group_subnets.sort()

            # See if anything changed.
            if (
                matching_groups["db_subnet_group_name"] != group_name
                or matching_groups["db_subnet_group_description"] != group_description
                or existing_subnets != group_subnets
            ):
                if not module.check_mode:
                    # Modify existing group.
                    try:
                        connection.modify_db_subnet_group(
                            aws_retry=True,
                            DBSubnetGroupName=group_name,
                            DBSubnetGroupDescription=group_description,
                            SubnetIds=group_subnets,
                        )
                    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                        module.fail_json_aws(e, "Failed to update a subnet group.")
                subnet_update = True
        else:
            if not module.check_mode:
                try:
                    connection.create_db_subnet_group(
                        aws_retry=True,
                        DBSubnetGroupName=group_name,
                        DBSubnetGroupDescription=group_description,
                        SubnetIds=group_subnets,
                        Tags=_tags,
                    )
                except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
                    module.fail_json_aws(e, "Failed to create a new subnet group.")
            subnet_update = True
    elif state == "absent":
        if not module.check_mode:
            try:
                connection.delete_db_subnet_group(aws_retry=True, DBSubnetGroupName=group_name)
            except is_boto3_error_code("DBSubnetGroupNotFoundFault"):
                module.exit_json(**result)
            except (
                botocore.exceptions.BotoCoreError,
                botocore.exceptions.ClientError,
            ) as e:  # pylint: disable=duplicate-except
                module.fail_json_aws(e, "Failed to delete a subnet group.")
        else:
            subnet_group = get_subnet_group(connection, module)
            if subnet_group:
                subnet_update = True
            result = create_result(subnet_update, subnet_group)
            module.exit_json(**result)

        subnet_update = True

    subnet_group = get_subnet_group(connection, module)
    changed = tags_update or subnet_update
    result = create_result(changed, subnet_group)
    module.exit_json(**result)


if __name__ == "__main__":
    main()
